@@ -140,6 +140,8 @@ B_DB_INGRES::B_DB_INGRES(JCR *jcr,
                          const char *db_socket,
                          bool mult_db_connections,
                          bool disable_batch_insert,
+                         bool try_reconnect,
+                         bool exit_on_fatal,
                          bool need_private)
 {
    B_DB_INGRES *mdb;
@@ -200,6 +202,8 @@ B_DB_INGRES::B_DB_INGRES(JCR *jcr,
    esc_obj = get_pool_memory(PM_FNAME);
    m_allow_transactions = mult_db_connections;
    m_is_private = need_private;
+   m_try_reconnect = try_reconnect;
+   m_exit_on_fatal = exit_on_fatal;
 
    /*
     * Initialize the private members.
@@ -543,8 +547,8 @@ bool B_DB_INGRES::sql_query(const char *query, int flags)
       /*
        * We are starting a new query.  reset everything.
        */
-      m_num_rows     = -1;
-      m_row_number   = -1;
+      m_num_rows = -1;
+      m_row_number = -1;
       m_field_number = -1;
 
       if (m_result) {
@@ -1010,6 +1014,8 @@ extern "C" B_DB CATS_IMP_EXP *backend_instantiate(JCR *jcr,
                                                   const char *db_socket,
                                                   bool mult_db_connections,
                                                   bool disable_batch_insert,
+                                                  bool try_reconnect,
+                                                  bool exit_on_fatal,
                                                   bool need_private)
 #else
 B_DB *db_init_database(JCR *jcr,
@@ -1022,6 +1028,8 @@ B_DB *db_init_database(JCR *jcr,
                        const char *db_socket,
                        bool mult_db_connections,
                        bool disable_batch_insert,
+                       bool try_reconnect,
+                       bool exit_on_fatal,
                        bool need_private)
 #endif
 {
@@ -1062,6 +1070,8 @@ B_DB *db_init_database(JCR *jcr,
                          db_socket,
                          mult_db_connections,
                          disable_batch_insert,
+                         try_reconnect,
+                         exit_on_fatal,
                          need_private));
 
 bail_out:
