@@ -3,7 +3,7 @@
 
    Copyright (C) 2002-2010 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -52,40 +52,153 @@ const char *cleanup_running_job =
 
 /* For sql_update.c db_update_stats */
 const char *fill_jobhisto =
-        "INSERT INTO JobHisto ("
-           "JobId, Job, Name, Type, Level, ClientId, JobStatus, "
-           "SchedTime, StartTime, EndTime, RealEndTime, JobTDate, "
-           "VolSessionId, VolSessionTime, JobFiles, JobBytes, ReadBytes, "
-           "JobErrors, JobMissingFiles, PoolId, FileSetId, PriorJobId, "
-           "PurgedFiles, HasBase, Reviewed, Comment ) "
-        "SELECT "
-           "JobId, Job, Name, Type, Level, ClientId, JobStatus, "
-           "SchedTime, StartTime, EndTime, RealEndTime, JobTDate, "
-           "VolSessionId, VolSessionTime, JobFiles, JobBytes, ReadBytes, "
-           "JobErrors, JobMissingFiles, PoolId, FileSetId, PriorJobId, "
-           "PurgedFiles, HasBase, Reviewed, Comment "
-          "FROM Job "
-         "WHERE JobStatus IN ('T','W','f','A','E') "
-           "AND NOT EXISTS "
-                "(SELECT JobHisto.JobId "
-                   "FROM JobHisto WHERE JobHisto.Jobid=Job.JobId) "
-           "AND JobTDate < %s ";
+   "INSERT INTO JobHisto ("
+      "JobId, Job, Name, Type, Level, ClientId, JobStatus, "
+      "SchedTime, StartTime, EndTime, RealEndTime, JobTDate, "
+      "VolSessionId, VolSessionTime, JobFiles, JobBytes, ReadBytes, "
+      "JobErrors, JobMissingFiles, PoolId, FileSetId, PriorJobId, "
+      "PurgedFiles, HasBase, Reviewed, Comment ) "
+   "SELECT "
+      "JobId, Job, Name, Type, Level, ClientId, JobStatus, "
+      "SchedTime, StartTime, EndTime, RealEndTime, JobTDate, "
+      "VolSessionId, VolSessionTime, JobFiles, JobBytes, ReadBytes, "
+      "JobErrors, JobMissingFiles, PoolId, FileSetId, PriorJobId, "
+      "PurgedFiles, HasBase, Reviewed, Comment "
+     "FROM Job "
+    "WHERE JobStatus IN ('T','W','f','A','E') "
+      "AND NOT EXISTS "
+           "(SELECT JobHisto.JobId "
+              "FROM JobHisto WHERE JobHisto.Jobid=Job.JobId) "
+      "AND JobTDate < %s ";
 
 /* For ua_update.c */
 const char *list_pool = "SELECT * FROM Pool WHERE PoolId=%s";
 
-/* For ua_dotcmds.c */
-const char *client_backups =
-   "SELECT DISTINCT Job.JobId,Client.Name as Client,Level,StartTime,"
-   "JobFiles,JobBytes,VolumeName,MediaType,FileSet,Media.Enabled as Enabled"
-   " FROM Client,Job,JobMedia,Media,FileSet"
-   " WHERE Client.Name='%s'"
-   " AND FileSet='%s'"
-   " AND Client.ClientId=Job.ClientId"
-   " AND JobStatus IN ('T','W') AND Type='B'"
-   " AND JobMedia.JobId=Job.JobId AND JobMedia.MediaId=Media.MediaId"
-   " AND Job.FileSetId=FileSet.FileSetId"
-   " ORDER BY Job.StartTime";
+/*
+ * For sql_list.c and ua_output.c
+ *
+ * "WHERE JobId > 0" is a dummy where clause,
+ *                   to make it easier to extend the filter
+ *                   (all filters starts with "AND ...")
+ * JOIN with Media is required for filter to Media.Volumename.
+ */
+const char *list_jobs =
+   "SELECT DISTINCT Job.JobId,Job.Name, Client.Name as Client, "
+   "Job.StartTime,Job.Type,Job.Level,Job.JobFiles,Job.JobBytes,Job.JobStatus "
+   "FROM Job "
+   "LEFT JOIN Client ON Client.ClientId=Job.ClientId "
+   "LEFT JOIN JobMedia ON JobMedia.JobId=Job.JobId "
+   "LEFT JOIN Media ON JobMedia.MediaId=Media.MediaId "
+   "LEFT JOIN FileSet ON FileSet.FileSetId=Job.FileSetId "
+   "WHERE Job.JobId > 0 "
+   "%s "
+   "ORDER BY StartTime%s";
+
+const char *list_jobs_last =
+   "SELECT DISTINCT Job.JobId,Job.Name, Client.Name as Client, "
+   "Job.StartTime,Job.Type,Job.Level,Job.JobFiles,Job.JobBytes,Job.JobStatus "
+   "FROM Job "
+   "LEFT JOIN Client ON Client.ClientId=Job.ClientId "
+   "LEFT JOIN JobMedia ON JobMedia.JobId=Job.JobId "
+   "LEFT JOIN Media ON JobMedia.MediaId=Media.MediaId "
+   "LEFT JOIN FileSet ON FileSet.FileSetId=Job.FileSetId "
+   "INNER JOIN ( "
+      "SELECT MAX(Job.JobId) as MaxJobId "
+      "FROM Job "
+      "LEFT JOIN Client ON Client.ClientId=Job.ClientId "
+      "LEFT JOIN Pool ON Pool.PoolId=Job.PoolId "
+      "LEFT JOIN JobMedia ON JobMedia.JobId=Job.JobId "
+      "LEFT JOIN Media ON JobMedia.MediaId=Media.MediaId "
+      "LEFT JOIN FileSet ON FileSet.FileSetId=Job.FileSetId "
+      "WHERE Job.JobId > 0 "
+      "%s "
+      "GROUP BY Job.Name "
+   ") LastJob "
+   "ON Job.JobId = LastJob.MaxJobId "
+   "ORDER BY StartTime%s";
+
+const char *list_jobs_count =
+   "SELECT DISTINCT COUNT(DISTINCT Job.JobId) as count "
+   "FROM Job "
+   "LEFT JOIN Client ON Client.ClientId=Job.ClientId "
+   "LEFT JOIN JobMedia ON JobMedia.JobId=Job.JobId "
+   "LEFT JOIN Media ON JobMedia.MediaId=Media.MediaId "
+   "LEFT JOIN FileSet ON FileSet.FileSetId=Job.FileSetId "
+   "WHERE Job.JobId > 0 "
+   "%s "
+   "%s";
+
+const char *list_jobs_long =
+   "SELECT DISTINCT Job.JobId, Job.Job, Job.Name, Job.PurgedFiles, Job.Type, Job.Level, "
+   "Job.ClientId, Client.Name as Client, Job.JobStatus, Job.SchedTime, Job.StartTime, "
+   "Job.EndTime, Job.RealEndTime, Job.JobTDate, Job.VolSessionId, Job.VolSessionTime, "
+   "Job.JobFiles, Job.JobBytes, Job.JobErrors, Job.JobMissingFiles, Job.PoolId, "
+   "Pool.Name as PoolName, Job.PriorJobId, Job.FileSetId, FileSet.FileSet "
+   "FROM Job "
+   "LEFT JOIN Client ON Client.ClientId=Job.ClientId "
+   "LEFT JOIN Pool ON Pool.PoolId=Job.PoolId "
+   "LEFT JOIN JobMedia ON JobMedia.JobId=Job.JobId "
+   "LEFT JOIN Media ON JobMedia.MediaId=Media.MediaId "
+   "LEFT JOIN FileSet ON FileSet.FileSetId=Job.FileSetId "
+   "WHERE Job.JobId > 0 "
+   "%s "
+   "ORDER BY StartTime%s";
+
+const char *list_jobs_long_last =
+   "SELECT DISTINCT Job.JobId, Job.Job, Job.Name, Job.PurgedFiles, Job.Type, Job.Level, "
+   "Job.ClientId, Client.Name as Client, Job.JobStatus, Job.SchedTime, Job.StartTime, "
+   "Job.EndTime, Job.RealEndTime, Job.JobTDate, Job.VolSessionId, Job.VolSessionTime, "
+   "Job.JobFiles, Job.JobBytes, Job.JobErrors, Job.JobMissingFiles, Job.PoolId, "
+   "Pool.Name as PoolName, Job.PriorJobId, Job.FileSetId, FileSet.FileSet "
+   "FROM Job "
+   "LEFT JOIN Client ON Client.ClientId=Job.ClientId "
+   "LEFT JOIN Pool ON Pool.PoolId=Job.PoolId "
+   "LEFT JOIN JobMedia ON JobMedia.JobId=Job.JobId "
+   "LEFT JOIN Media ON JobMedia.MediaId=Media.MediaId "
+   "LEFT JOIN FileSet ON FileSet.FileSetId=Job.FileSetId "
+   "INNER JOIN ( "
+      "SELECT MAX(Job.JobId) as MaxJobId "
+      "FROM Job "
+      "LEFT JOIN Client ON Client.ClientId=Job.ClientId "
+      "LEFT JOIN Pool ON Pool.PoolId=Job.PoolId "
+      "LEFT JOIN JobMedia ON JobMedia.JobId=Job.JobId "
+      "LEFT JOIN Media ON JobMedia.MediaId=Media.MediaId "
+      "LEFT JOIN FileSet ON FileSet.FileSetId=Job.FileSetId "
+      "WHERE Job.JobId > 0 "
+      "%s "
+      "GROUP BY Job.Name "
+   ") LastJob "
+   "ON Job.JobId = LastJob.MaxJobId "
+   "ORDER BY StartTime%s";
+
+const char *get_jobstatus_details =
+   "SELECT DISTINCT "
+     "JobStatus,JobStatusLong,Severity,"
+      /*
+       * JS_Error 'e'  Non-fatal error
+       * JS_FatalError 'f' Fatal error
+       * JS_Canceled 'A'
+       * JS_ErrorTerminated 'E'
+       * JS_Terminated 'T'
+       * JS_Warnings 'W'
+       */
+      "CASE "
+         /* Ok */
+         "WHEN JobStatus in ('T') THEN '0' "
+         /* Warning */
+         "WHEN JobStatus in ('W') THEN '1' "
+         /* Critical */
+         "WHEN JobStatus in ('e', 'f', 'A', 'E') THEN '2' "
+      "END as ExitLevel, "
+      "CASE "
+         "WHEN JobStatus in ('T') THEN 'Ok' "
+         "WHEN JobStatus in ('W') THEN 'Warning' "
+         "WHEN JobStatus in ('e', 'f', 'A', 'E') THEN 'Error' "
+      "END as ExitStatus "
+   "FROM Status "
+   /* optional WHERE */
+   "%s "
+   "ORDER BY JobStatus";
 
 /* ====== ua_prune.c */
 
@@ -95,9 +208,22 @@ const char *sel_JobMedia =
    "AND Job.JobTDate<%s";
 
 /* Delete temp tables and indexes  */
+static const char *drop_deltabs_default =
+   "DROP TABLE DelCandidates";
+
 const char *drop_deltabs[] = {
-   "DROP TABLE DelCandidates",
-   NULL};
+   /* MySQL */
+   "DROP TABLE IF EXISTS DelCandidates",
+
+   /* Postgresql */
+   "DROP TABLE IF EXISTS DelCandidates",
+
+   /* SQLite3 */
+   drop_deltabs_default,
+
+   /* Ingres */
+   drop_deltabs_default
+};
 
 const char *create_delindex = "CREATE INDEX DelInx1 ON DelCandidates (JobId)";
 
@@ -132,7 +258,7 @@ const char *uar_del_temp  = "DROP TABLE temp";
 const char *uar_del_temp1 = "DROP TABLE temp1";
 
 const char *uar_last_full =
-   "INSERT INTO temp1 SELECT Job.JobId,JobTdate "
+   "INSERT INTO temp1 SELECT Job.JobId,Job.JobTdate "
    "FROM Job,JobMedia,Media,FileSet "
    "WHERE Job.ClientId=%s "
    "AND Job.StartTime < '%s' "
@@ -146,7 +272,7 @@ const char *uar_last_full =
    "ORDER BY Job.JobTDate DESC LIMIT 1";
 
 const char *uar_last_full_no_pool =
-   "INSERT INTO temp1 SELECT Job.JobId,JobTdate "
+   "INSERT INTO temp1 SELECT Job.JobId,Job.JobTdate "
    "FROM Job,FileSet "
    "WHERE Job.ClientId=%s "
    "AND Job.StartTime < '%s' "
@@ -156,21 +282,16 @@ const char *uar_last_full_no_pool =
    "ORDER BY Job.JobTDate DESC LIMIT 1";
 
 const char *uar_full =
-   "INSERT INTO temp SELECT Job.JobId,Job.JobTDate,"
-   "Job.ClientId,Job.Level,Job.JobFiles,Job.JobBytes,"
-   "StartTime,VolumeName,JobMedia.StartFile,VolSessionId,VolSessionTime "
+   "INSERT INTO temp SELECT Job.JobId,Job.JobTDate,Job.StartTime "
    "FROM temp1,Job,JobMedia,Media "
    "WHERE temp1.JobId=Job.JobId "
    "AND Level='F' AND JobStatus IN ('T','W') AND Type='B' "
-   "AND Media.Enabled=1 "
    "AND JobMedia.JobId=Job.JobId "
+   "AND Media.Enabled=1 "
    "AND JobMedia.MediaId=Media.MediaId";
 
 const char *uar_dif =
-   "INSERT INTO temp SELECT Job.JobId,Job.JobTDate,Job.ClientId,"
-   "Job.Level,Job.JobFiles,Job.JobBytes,"
-   "Job.StartTime,Media.VolumeName,JobMedia.StartFile,"
-   "Job.VolSessionId,Job.VolSessionTime "
+   "INSERT INTO temp SELECT Job.JobId,Job.JobTDate,Job.StartTime "
    "FROM Job,JobMedia,Media,FileSet "
    "WHERE Job.JobTDate>%s AND Job.StartTime<'%s' "
    "AND Job.ClientId=%s "
@@ -184,27 +305,18 @@ const char *uar_dif =
    "ORDER BY Job.JobTDate DESC LIMIT 1";
 
 const char *uar_inc =
-   "INSERT INTO temp SELECT Job.JobId,Job.JobTDate,Job.ClientId,"
-   "Job.Level,Job.JobFiles,Job.JobBytes,"
-   "Job.StartTime,Media.VolumeName,JobMedia.StartFile,"
-   "Job.VolSessionId,Job.VolSessionTime "
+   "INSERT INTO temp SELECT Job.JobId,Job.JobTDate,Job.StartTime "
    "FROM Job,JobMedia,Media,FileSet "
    "WHERE Job.JobTDate>%s AND Job.StartTime<'%s' "
    "AND Job.ClientId=%s "
-   "AND Media.Enabled=1 "
    "AND JobMedia.JobId=Job.JobId "
+   "AND Media.Enabled=1 "
    "AND JobMedia.MediaId=Media.MediaId "
    "AND Job.Level='I' AND JobStatus IN ('T','W') AND Type='B' "
    "AND Job.FileSetId=FileSet.FileSetId "
    "AND FileSet.FileSet='%s' "
    "%s";
 
-const char *uar_list_temp =
-   "SELECT DISTINCT JobId,Level,JobFiles,JobBytes,StartTime,VolumeName"
-   " FROM temp"
-   " ORDER BY StartTime ASC";
-
-
 const char *uar_sel_jobid_temp =
    "SELECT DISTINCT JobId,StartTime FROM temp ORDER BY StartTime ASC";
 
@@ -212,12 +324,28 @@ const char *uar_sel_all_temp1 = "SELECT * FROM temp1";
 
 const char *uar_sel_all_temp = "SELECT * FROM temp";
 
+const char *uar_sel_jobid_copies =
+   "SELECT DISTINCT JobId,StartTime "
+   "FROM Job "
+   "WHERE Type = 'C' "
+   "AND (PriorJobId IN (%s)) "
+   "ORDER BY StartTime ASC";
 
+const char *uar_list_jobs_by_idlist =
+   "SELECT DISTINCT Job.JobId,Job.Level,Job.JobFiles,Job.JobBytes,"
+   "Job.StartTime,Media.VolumeName "
+   "FROM Job,JobMedia,Media "
+   "WHERE Job.JobId IN (%s) "
+   "AND JobMedia.JobId=Job.JobId "
+   "AND Media.Enabled=1 "
+   "AND JobMedia.MediaId=Media.MediaId "
+   "ORDER BY StartTime ASC";
 
 /* Select FileSet names for this Client */
 const char *uar_sel_fileset =
-   "SELECT DISTINCT FileSet.FileSet FROM Job,"
-   "Client,FileSet WHERE Job.FileSetId=FileSet.FileSetId "
+   "SELECT DISTINCT FileSet.FileSet "
+   "FROM Job,Client,FileSet "
+   "WHERE Job.FileSetId=FileSet.FileSetId "
    "AND Job.ClientId=%s AND Client.ClientId=%s "
    "ORDER BY FileSet.FileSet";
 
@@ -277,31 +405,31 @@ const char *uar_jobid_fileindex_from_table =
  * we need an extra join using JobTDate.
  */
 static const char *select_recent_version_with_basejob_default =
-"SELECT FileId, Job.JobId AS JobId, FileIndex, File.PathId AS PathId, "
-       "File.FilenameId AS FilenameId, LStat, MD5, DeltaSeq, "
-       "Job.JobTDate AS JobTDate "
-"FROM Job, File, ( "
-    "SELECT MAX(JobTDate) AS JobTDate, PathId, FilenameId "
-      "FROM ( "
-        "SELECT JobTDate, PathId, FilenameId "   /* Get all normal files */
-          "FROM File JOIN Job USING (JobId) "    /* from selected backup */
-         "WHERE File.JobId IN (%s) "
-          "UNION ALL "
-        "SELECT JobTDate, PathId, FilenameId "   /* Get all files from */
-          "FROM BaseFiles "                      /* BaseJob */
-               "JOIN File USING (FileId) "
-               "JOIN Job  ON    (BaseJobId = Job.JobId) "
-         "WHERE BaseFiles.JobId IN (%s) "        /* Use Max(JobTDate) to find */
-       ") AS tmp "
-       "GROUP BY PathId, FilenameId "            /* the latest file version */
-    ") AS T1 "
-"WHERE (Job.JobId IN ( "  /* Security, we force JobId to be valid */
-        "SELECT DISTINCT BaseJobId FROM BaseFiles WHERE JobId IN (%s)) "
-        "OR Job.JobId IN (%s)) "
-  "AND T1.JobTDate = Job.JobTDate " /* Join on JobTDate to get the orginal */
-  "AND Job.JobId = File.JobId "     /* Job/File record */
-  "AND T1.PathId = File.PathId "
-  "AND T1.FilenameId = File.FilenameId";
+   "SELECT FileId, Job.JobId AS JobId, FileIndex, File.PathId AS PathId, "
+          "File.FilenameId AS FilenameId, LStat, MD5, DeltaSeq, "
+          "Job.JobTDate AS JobTDate "
+   "FROM Job, File, ( "
+       "SELECT MAX(JobTDate) AS JobTDate, PathId, FilenameId "
+         "FROM ( "
+           "SELECT JobTDate, PathId, FilenameId "   /* Get all normal files */
+             "FROM File JOIN Job USING (JobId) "    /* from selected backup */
+            "WHERE File.JobId IN (%s) "
+             "UNION ALL "
+           "SELECT JobTDate, PathId, FilenameId "   /* Get all files from */
+             "FROM BaseFiles "                      /* BaseJob */
+                  "JOIN File USING (FileId) "
+                  "JOIN Job  ON    (BaseJobId = Job.JobId) "
+            "WHERE BaseFiles.JobId IN (%s) "        /* Use Max(JobTDate) to find */
+          ") AS tmp "
+          "GROUP BY PathId, FilenameId "            /* the latest file version */
+       ") AS T1 "
+   "WHERE (Job.JobId IN ( "  /* Security, we force JobId to be valid */
+           "SELECT DISTINCT BaseJobId FROM BaseFiles WHERE JobId IN (%s)) "
+           "OR Job.JobId IN (%s)) "
+     "AND T1.JobTDate = Job.JobTDate " /* Join on JobTDate to get the orginal */
+     "AND Job.JobId = File.JobId "     /* Job/File record */
+     "AND T1.PathId = File.PathId "
+     "AND T1.FilenameId = File.FilenameId";
 
 const char *select_recent_version_with_basejob[] = {
    /* MySQL */
@@ -344,31 +472,31 @@ const char *select_recent_version_with_basejob[] = {
  * This control should be reset for each new file
  */
 static const char *select_recent_version_with_basejob_and_delta_default =
-"SELECT FileId, Job.JobId AS JobId, FileIndex, File.PathId AS PathId, "
-       "File.FilenameId AS FilenameId, LStat, MD5, File.DeltaSeq AS DeltaSeq, "
-       "Job.JobTDate AS JobTDate "
-"FROM Job, File, ( "
-    "SELECT MAX(JobTDate) AS JobTDate, PathId, FilenameId, DeltaSeq "
-      "FROM ( "
-       "SELECT JobTDate, PathId, FilenameId, DeltaSeq " /*Get all normal files*/
-         "FROM File JOIN Job USING (JobId) "          /* from selected backup */
-        "WHERE File.JobId IN (%s) "
-         "UNION ALL "
-       "SELECT JobTDate, PathId, FilenameId, DeltaSeq " /*Get all files from */
-         "FROM BaseFiles "                            /* BaseJob */
-              "JOIN File USING (FileId) "
-              "JOIN Job  ON    (BaseJobId = Job.JobId) "
-        "WHERE BaseFiles.JobId IN (%s) "        /* Use Max(JobTDate) to find */
-       ") AS tmp "
-       "GROUP BY PathId, FilenameId, DeltaSeq "    /* the latest file version */
-    ") AS T1 "
-"WHERE (Job.JobId IN ( "  /* Security, we force JobId to be valid */
-        "SELECT DISTINCT BaseJobId FROM BaseFiles WHERE JobId IN (%s)) "
-        "OR Job.JobId IN (%s)) "
-  "AND T1.JobTDate = Job.JobTDate " /* Join on JobTDate to get the orginal */
-  "AND Job.JobId = File.JobId "     /* Job/File record */
-  "AND T1.PathId = File.PathId "
-  "AND T1.FilenameId = File.FilenameId";
+   "SELECT FileId, Job.JobId AS JobId, FileIndex, File.PathId AS PathId, "
+          "File.FilenameId AS FilenameId, LStat, MD5, File.DeltaSeq AS DeltaSeq, "
+          "Job.JobTDate AS JobTDate "
+   "FROM Job, File, ( "
+       "SELECT MAX(JobTDate) AS JobTDate, PathId, FilenameId, DeltaSeq "
+         "FROM ( "
+          "SELECT JobTDate, PathId, FilenameId, DeltaSeq " /*Get all normal files*/
+            "FROM File JOIN Job USING (JobId) "          /* from selected backup */
+           "WHERE File.JobId IN (%s) "
+            "UNION ALL "
+          "SELECT JobTDate, PathId, FilenameId, DeltaSeq " /*Get all files from */
+            "FROM BaseFiles "                            /* BaseJob */
+                 "JOIN File USING (FileId) "
+                 "JOIN Job  ON    (BaseJobId = Job.JobId) "
+           "WHERE BaseFiles.JobId IN (%s) "        /* Use Max(JobTDate) to find */
+          ") AS tmp "
+          "GROUP BY PathId, FilenameId, DeltaSeq "    /* the latest file version */
+       ") AS T1 "
+   "WHERE (Job.JobId IN ( "  /* Security, we force JobId to be valid */
+           "SELECT DISTINCT BaseJobId FROM BaseFiles WHERE JobId IN (%s)) "
+           "OR Job.JobId IN (%s)) "
+     "AND T1.JobTDate = Job.JobTDate " /* Join on JobTDate to get the orginal */
+     "AND Job.JobId = File.JobId "     /* Job/File record */
+     "AND T1.PathId = File.PathId "
+     "AND T1.FilenameId = File.FilenameId";
 
 const char *select_recent_version_with_basejob_and_delta[] = {
    /* MySQL */
@@ -399,20 +527,20 @@ const char *select_recent_version_with_basejob_and_delta[] = {
  * We don't handle Delta with BaseJobs, they have only Full files
  */
 static const char *select_recent_version_default =
-  "SELECT j1.JobId AS JobId, f1.FileId AS FileId, f1.FileIndex AS FileIndex, "
-          "f1.PathId AS PathId, f1.FilenameId AS FilenameId, "
-          "f1.LStat AS LStat, f1.MD5 AS MD5, j1.JobTDate "
-     "FROM ( "     /* Choose the last version for each Path/Filename */
-       "SELECT max(JobTDate) AS JobTDate, PathId, FilenameId "
-         "FROM File JOIN Job USING (JobId) "
-        "WHERE File.JobId IN (%s) "
-       "GROUP BY PathId, FilenameId "
-     ") AS t1, Job AS j1, File AS f1 "
-    "WHERE t1.JobTDate = j1.JobTDate "
-      "AND j1.JobId IN (%s) "
-      "AND t1.FilenameId = f1.FilenameId "
-      "AND t1.PathId = f1.PathId "
-      "AND j1.JobId = f1.JobId";
+   "SELECT j1.JobId AS JobId, f1.FileId AS FileId, f1.FileIndex AS FileIndex, "
+           "f1.PathId AS PathId, f1.FilenameId AS FilenameId, "
+           "f1.LStat AS LStat, f1.MD5 AS MD5, j1.JobTDate "
+      "FROM ( "     /* Choose the last version for each Path/Filename */
+        "SELECT max(JobTDate) AS JobTDate, PathId, FilenameId "
+          "FROM File JOIN Job USING (JobId) "
+         "WHERE File.JobId IN (%s) "
+        "GROUP BY PathId, FilenameId "
+      ") AS t1, Job AS j1, File AS f1 "
+     "WHERE t1.JobTDate = j1.JobTDate "
+       "AND j1.JobId IN (%s) "
+       "AND t1.FilenameId = f1.FilenameId "
+       "AND t1.PathId = f1.PathId "
+       "AND j1.JobId = f1.JobId";
 
 const char *select_recent_version[] = {
    /* MySQL */
@@ -436,14 +564,14 @@ const char *select_recent_version[] = {
  * 5.0 and 5.1 are unable to run further queries in this mode
  */
 static const char *create_temp_accurate_jobids_default =
- "CREATE TABLE btemp3%s AS "
-    "SELECT JobId, StartTime, EndTime, JobTDate, PurgedFiles "
-      "FROM Job JOIN FileSet USING (FileSetId) "
-     "WHERE ClientId = %s "
-       "AND Level='F' AND JobStatus IN ('T','W') AND Type='B' "
-       "AND StartTime<'%s' "
-       "AND FileSet.FileSet=(SELECT FileSet FROM FileSet WHERE FileSetId = %s) "
-     "ORDER BY Job.JobTDate DESC LIMIT 1";
+   "CREATE TABLE btemp3%s AS "
+      "SELECT JobId, StartTime, EndTime, JobTDate, PurgedFiles "
+        "FROM Job JOIN FileSet USING (FileSetId) "
+       "WHERE ClientId = %s "
+         "AND Level='F' AND JobStatus IN ('T','W') AND Type='B' "
+         "AND StartTime<'%s' "
+         "AND FileSet.FileSet=(SELECT FileSet FROM FileSet WHERE FileSetId = %s) "
+       "ORDER BY Job.JobTDate DESC LIMIT 1";
 
 const char *create_temp_accurate_jobids[] = {
    /* Mysql */
@@ -577,19 +705,19 @@ const char *create_deltabs[] = {
 /* Select the first available Copy Job that must be upgraded to a Backup job when the original backup job is expired. */
 
 static const char *uap_upgrade_copies_oldest_job_default =
-"CREATE TEMPORARY TABLE cpy_tmp AS "
-       "SELECT MIN(JobId) AS JobId FROM Job "     /* Choose the oldest job */
-        "WHERE Type='%c' "                        /* JT_JOB_COPY */
-          "AND ( PriorJobId IN (%s) "             /* JobId selection */
-              "OR "
-               " PriorJobId IN ( "
-                  "SELECT PriorJobId "
-                    "FROM Job "
-                   "WHERE JobId IN (%s) "         /* JobId selection */
-                    " AND Type='B' "
+   "CREATE TEMPORARY TABLE cpy_tmp AS "
+          "SELECT MIN(JobId) AS JobId FROM Job "     /* Choose the oldest job */
+           "WHERE Type='%c' "                        /* JT_JOB_COPY */
+             "AND ( PriorJobId IN (%s) "             /* JobId selection */
+                 "OR "
+                  " PriorJobId IN ( "
+                     "SELECT PriorJobId "
+                       "FROM Job "
+                      "WHERE JobId IN (%s) "         /* JobId selection */
+                       " AND Type='B' "
+                    ") "
                  ") "
-              ") "
-          "GROUP BY PriorJobId ";           /* one result per copy */
+             "GROUP BY PriorJobId ";           /* one result per copy */
 
 const char *uap_upgrade_copies_oldest_job[] = {
    /* Mysql */
@@ -668,57 +796,25 @@ const char *uar_create_temp[] = {
    "CREATE TEMPORARY TABLE temp ("
    "JobId INTEGER UNSIGNED NOT NULL,"
    "JobTDate BIGINT UNSIGNED,"
-   "ClientId INTEGER UNSIGNED,"
-   "Level CHAR,"
-   "JobFiles INTEGER UNSIGNED,"
-   "JobBytes BIGINT UNSIGNED,"
-   "StartTime TEXT,"
-   "VolumeName TEXT,"
-   "StartFile INTEGER UNSIGNED,"
-   "VolSessionId INTEGER UNSIGNED,"
-   "VolSessionTime INTEGER UNSIGNED)",
+   "StartTime TEXT)",
 
    /* Postgresql */
    "CREATE TEMPORARY TABLE temp ("
    "JobId INTEGER NOT NULL,"
    "JobTDate BIGINT,"
-   "ClientId INTEGER,"
-   "Level CHAR,"
-   "JobFiles INTEGER,"
-   "JobBytes BIGINT,"
-   "StartTime TEXT,"
-   "VolumeName TEXT,"
-   "StartFile INTEGER,"
-   "VolSessionId INTEGER,"
-   "VolSessionTime INTEGER)",
+   "StartTime TEXT)",
 
    /* SQLite3 */
    "CREATE TEMPORARY TABLE temp ("
    "JobId INTEGER UNSIGNED NOT NULL,"
    "JobTDate BIGINT UNSIGNED,"
-   "ClientId INTEGER UNSIGNED,"
-   "Level CHAR,"
-   "JobFiles INTEGER UNSIGNED,"
-   "JobBytes BIGINT UNSIGNED,"
-   "StartTime TEXT,"
-   "VolumeName TEXT,"
-   "StartFile INTEGER UNSIGNED,"
-   "VolSessionId INTEGER UNSIGNED,"
-   "VolSessionTime INTEGER UNSIGNED)",
+   "StartTime TEXT)",
 
    /* Ingres */
    "DECLARE GLOBAL TEMPORARY TABLE temp ("
    "JobId INTEGER NOT NULL,"
    "JobTDate BIGINT,"
-   "ClientId INTEGER,"
-   "Level CHAR(1),"
-   "JobFiles INTEGER,"
-   "JobBytes BIGINT,"
-   "StartTime TIMESTAMP WITHOUT TIME ZONE,"
-   "VolumeName VARBYTE(128),"
-   "StartFile INTEGER,"
-   "VolSessionId INTEGER,"
-   "VolSessionTime INTEGER) "
+   "StartTime TIMESTAMP WITHOUT TIME ZONE) "
    "ON COMMIT PRESERVE ROWS WITH NORECOVERY"
 };
 
@@ -829,18 +925,18 @@ const char *sql_get_max_connections[] = {
  *  version of a file in the same dataset.
  */
 const char *default_sql_bvfs_select =
-"CREATE TABLE %s AS "
-"SELECT File.JobId, File.FileIndex, File.FileId "
-"FROM Job, File, ( "
-    "SELECT MAX(JobTDate) AS JobTDate, PathId, FilenameId "
-       "FROM btemp%s GROUP BY PathId, FilenameId "
-    ") AS T1 JOIN Filename USING (FilenameId) "
-"WHERE T1.JobTDate = Job.JobTDate "
-  "AND Job.JobId = File.JobId "
-  "AND T1.PathId = File.PathId "
-  "AND T1.FilenameId = File.FilenameId "
-  "AND File.FileIndex > 0 "
-  "AND Job.JobId IN (SELECT DISTINCT JobId FROM btemp%s) ";
+   "CREATE TABLE %s AS "
+   "SELECT File.JobId, File.FileIndex, File.FileId "
+   "FROM Job, File, ( "
+       "SELECT MAX(JobTDate) AS JobTDate, PathId, FilenameId "
+          "FROM btemp%s GROUP BY PathId, FilenameId "
+       ") AS T1 JOIN Filename USING (FilenameId) "
+   "WHERE T1.JobTDate = Job.JobTDate "
+     "AND Job.JobId = File.JobId "
+     "AND T1.PathId = File.PathId "
+     "AND T1.FilenameId = File.FilenameId "
+     "AND File.FileIndex > 0 "
+     "AND Job.JobId IN (SELECT DISTINCT JobId FROM btemp%s) ";
 
 const char *sql_bvfs_select[] = {
    /* Mysql */
@@ -865,33 +961,33 @@ const char *sql_bvfs_select[] = {
 };
 
 static const char *sql_bvfs_list_files_default =
-"SELECT 'F', T1.PathId, T1.FilenameId, Filename.Name, "
-        "File.JobId, File.LStat, File.FileId "
-"FROM Job, File, ( "
-    "SELECT MAX(JobTDate) AS JobTDate, PathId, FilenameId "
-      "FROM ( "
-        "SELECT JobTDate, PathId, FilenameId "
-          "FROM File JOIN Job USING (JobId) "
-         "WHERE File.JobId IN (%s) AND PathId = %s "
-          "UNION ALL "
-        "SELECT JobTDate, PathId, FilenameId "
-          "FROM BaseFiles "
-               "JOIN File USING (FileId) "
-               "JOIN Job  ON    (BaseJobId = Job.JobId) "
-         "WHERE BaseFiles.JobId IN (%s)   AND PathId = %s "
-       ") AS tmp GROUP BY PathId, FilenameId "
-     "LIMIT %lld OFFSET %lld"
-    ") AS T1 JOIN Filename USING (FilenameId) "
-"WHERE T1.JobTDate = Job.JobTDate "
-  "AND Job.JobId = File.JobId "
-  "AND T1.PathId = File.PathId "
-  "AND T1.FilenameId = File.FilenameId "
-  "AND Filename.Name != '' "
-  "AND File.FileIndex > 0 "
-  " %s "                     /* AND Name LIKE '' */
-  "AND (Job.JobId IN ( "
-        "SELECT DISTINCT BaseJobId FROM BaseFiles WHERE JobId IN (%s)) "
-       "OR Job.JobId IN (%s)) ";
+   "SELECT 'F', T1.PathId, T1.FilenameId, Filename.Name, "
+           "File.JobId, File.LStat, File.FileId "
+   "FROM Job, File, ( "
+       "SELECT MAX(JobTDate) AS JobTDate, PathId, FilenameId "
+         "FROM ( "
+           "SELECT JobTDate, PathId, FilenameId "
+             "FROM File JOIN Job USING (JobId) "
+            "WHERE File.JobId IN (%s) AND PathId = %s "
+             "UNION ALL "
+           "SELECT JobTDate, PathId, FilenameId "
+             "FROM BaseFiles "
+                  "JOIN File USING (FileId) "
+                  "JOIN Job  ON    (BaseJobId = Job.JobId) "
+            "WHERE BaseFiles.JobId IN (%s)   AND PathId = %s "
+          ") AS tmp GROUP BY PathId, FilenameId "
+        "LIMIT %lld OFFSET %lld"
+       ") AS T1 JOIN Filename USING (FilenameId) "
+   "WHERE T1.JobTDate = Job.JobTDate "
+     "AND Job.JobId = File.JobId "
+     "AND T1.PathId = File.PathId "
+     "AND T1.FilenameId = File.FilenameId "
+     "AND Filename.Name != '' "
+     "AND File.FileIndex > 0 "
+     " %s "                     /* AND Name LIKE '' */
+     "AND (Job.JobId IN ( "
+           "SELECT DISTINCT BaseJobId FROM BaseFiles WHERE JobId IN (%s)) "
+          "OR Job.JobId IN (%s)) ";
 
 const char *sql_bvfs_list_files[] = {
    /* Mysql */
@@ -900,24 +996,24 @@ const char *sql_bvfs_list_files[] = {
 
    /* JobId PathId JobId PathId WHERE? Filename? Limit Offset*/
    /* Postgresql */
- "SELECT Type, PathId, FilenameId, Name, JobId, LStat, FileId "
-  "FROM ("
-   "SELECT DISTINCT ON (FilenameId) 'F' as Type, PathId, T.FilenameId, "
-    "Filename.Name, JobId, LStat, FileId, FileIndex "
-     "FROM "
-         "(SELECT FileId, JobId, PathId, FilenameId, FileIndex, LStat, MD5 "
-            "FROM File WHERE JobId IN (%s) AND PathId = %s "
-           "UNION ALL "
-          "SELECT File.FileId, File.JobId, PathId, FilenameId, "
-                 "File.FileIndex, LStat, MD5 "
-            "FROM BaseFiles JOIN File USING (FileId) "
-           "WHERE BaseFiles.JobId IN (%s) AND File.PathId = %s "
-          ") AS T JOIN Job USING (JobId) JOIN Filename USING (FilenameId) "
-          " WHERE Filename.Name != '' "
-          " %s "               /* AND Name LIKE '' */
-     "ORDER BY FilenameId, StartTime DESC "
-   ") AS A WHERE A.FileIndex > 0 "
-   "LIMIT %lld OFFSET %lld ",
+   "SELECT Type, PathId, FilenameId, Name, JobId, LStat, FileId "
+    "FROM ("
+     "SELECT DISTINCT ON (FilenameId) 'F' as Type, PathId, T.FilenameId, "
+      "Filename.Name, JobId, LStat, FileId, FileIndex "
+       "FROM "
+           "(SELECT FileId, JobId, PathId, FilenameId, FileIndex, LStat, MD5 "
+              "FROM File WHERE JobId IN (%s) AND PathId = %s "
+             "UNION ALL "
+            "SELECT File.FileId, File.JobId, PathId, FilenameId, "
+                   "File.FileIndex, LStat, MD5 "
+              "FROM BaseFiles JOIN File USING (FileId) "
+             "WHERE BaseFiles.JobId IN (%s) AND File.PathId = %s "
+            ") AS T JOIN Job USING (JobId) JOIN Filename USING (FilenameId) "
+            " WHERE Filename.Name != '' "
+            " %s "               /* AND Name LIKE '' */
+       "ORDER BY FilenameId, StartTime DESC "
+     ") AS A WHERE A.FileIndex > 0 "
+     "LIMIT %lld OFFSET %lld ",
 
    /* SQLite */
    sql_bvfs_list_files_default,
