@@ -3,7 +3,7 @@
 
    Copyright (C) 2010-2012 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -49,6 +49,8 @@ B_DB *db_sql_get_non_pooled_connection(JCR *jcr,
                                        const char *db_socket,
                                        bool mult_db_connections,
                                        bool disable_batch_insert,
+                                       bool try_reconnect,
+                                       bool exit_on_fatal,
                                        bool need_private)
 {
    B_DB *mdb;
@@ -70,6 +72,8 @@ B_DB *db_sql_get_non_pooled_connection(JCR *jcr,
                           db_socket,
                           mult_db_connections,
                           disable_batch_insert,
+                          try_reconnect,
+                          exit_on_fatal,
                           need_private);
    if (mdb == NULL) {
       return NULL;
@@ -132,6 +136,8 @@ bool db_sql_pool_initialize(const char *db_drivername,
                             int db_port,
                             const char *db_socket,
                             bool disable_batch_insert,
+                            bool try_reconnect,
+                            bool exit_on_fatal,
                             int min_connections,
                             int max_connections,
                             int increment_connections,
@@ -200,7 +206,9 @@ bool db_sql_pool_initialize(const char *db_drivername,
                              db_port,
                              db_socket,
                              true,
-                             disable_batch_insert);
+                             disable_batch_insert,
+                             try_reconnect,
+                             exit_on_fatal);
       if (mdb == NULL) {
          Jmsg(NULL, M_FATAL, 0, "%s", _("Could not init database connection"));
          goto bail_out;
@@ -535,6 +543,8 @@ B_DB *db_sql_get_pooled_connection(JCR *jcr,
                                    const char *db_socket,
                                    bool mult_db_connections,
                                    bool disable_batch_insert,
+                                   bool try_reconnect,
+                                   bool exit_on_fatal,
                                    bool need_private)
 {
    int cnt = 0;
@@ -566,6 +576,8 @@ B_DB *db_sql_get_pooled_connection(JCR *jcr,
                                               db_socket,
                                               mult_db_connections,
                                               disable_batch_insert,
+                                              try_reconnect,
+                                              exit_on_fatal,
                                               need_private);
    }
 
@@ -625,6 +637,8 @@ B_DB *db_sql_get_pooled_connection(JCR *jcr,
                                                                db_socket,
                                                                mult_db_connections,
                                                                disable_batch_insert,
+                                                               try_reconnect,
+                                                               exit_on_fatal,
                                                                need_private);
                   goto bail_out;
                }
@@ -655,6 +669,8 @@ B_DB *db_sql_get_pooled_connection(JCR *jcr,
                                                    db_socket,
                                                    mult_db_connections,
                                                    disable_batch_insert,
+                                                   try_reconnect,
+                                                   exit_on_fatal,
                                                    need_private);
       goto bail_out;
    }
@@ -819,6 +835,8 @@ bool db_sql_pool_initialize(const char *db_drivername,
                             int db_port,
                             const char *db_socket,
                             bool disable_batch_insert,
+                            bool try_reconnect,
+                            bool exit_on_fatal,
                             int min_connections,
                             int max_connections,
                             int increment_connections,
@@ -858,6 +876,8 @@ B_DB *db_sql_get_pooled_connection(JCR *jcr,
                                    const char *db_socket,
                                    bool mult_db_connections,
                                    bool disable_batch_insert,
+                                   bool try_reconnect,
+                                   bool exit_on_fatal,
                                    bool need_private)
 {
    return db_sql_get_non_pooled_connection(jcr,
@@ -870,6 +890,8 @@ B_DB *db_sql_get_pooled_connection(JCR *jcr,
                                            db_socket,
                                            mult_db_connections,
                                            disable_batch_insert,
+                                           try_reconnect,
+                                           exit_on_fatal,
                                            need_private);
 }
 
