@@ -71,6 +71,8 @@ B_DB_SQLITE::B_DB_SQLITE(JCR *jcr,
                          const char *db_socket,
                          bool mult_db_connections,
                          bool disable_batch_insert,
+                         bool try_reconnect,
+                         bool exit_on_fatal,
                          bool need_private)
 {
    /*
@@ -108,6 +110,8 @@ B_DB_SQLITE::B_DB_SQLITE(JCR *jcr,
    esc_obj  = get_pool_memory(PM_FNAME);
    m_allow_transactions = mult_db_connections;
    m_is_private = need_private;
+   m_try_reconnect = try_reconnect;
+   m_exit_on_fatal = exit_on_fatal;
 
    /*
     * Initialize the private members.
@@ -153,8 +157,7 @@ bool B_DB_SQLITE::db_open_database(JCR *jcr)
 
    if ((errstat=rwl_init(&m_lock)) != 0) {
       berrno be;
-      Mmsg1(&errmsg, _("Unable to initialize DB lock. ERR=%s\n"),
-            be.bstrerror(errstat));
+      Mmsg1(errmsg, _("Unable to initialize DB lock. ERR=%s\n"), be.bstrerror(errstat));
       goto bail_out;
    }
 
@@ -168,8 +171,7 @@ bool B_DB_SQLITE::db_open_database(JCR *jcr)
    strcat(db_path, m_db_name);
    strcat(db_path, ".db");
    if (stat(db_path, &statbuf) != 0) {
-      Mmsg1(&errmsg, _("Database %s does not exist, please create it.\n"),
-         db_path);
+      Mmsg1(errmsg, _("Database %s does not exist, please create it.\n"), db_path);
       free(db_path);
       goto bail_out;
    }
@@ -190,8 +192,8 @@ bool B_DB_SQLITE::db_open_database(JCR *jcr)
       }
    }
    if (m_db_handle == NULL) {
-      Mmsg2(&errmsg, _("Unable to open Database=%s. ERR=%s\n"),
-         db_path, m_lowlevel_errmsg ? m_lowlevel_errmsg : _("unknown"));
+      Mmsg2(errmsg, _("Unable to open Database=%s. ERR=%s\n"),
+            db_path, m_lowlevel_errmsg ? m_lowlevel_errmsg : _("unknown"));
       free(db_path);
       goto bail_out;
    }
@@ -663,6 +665,8 @@ extern "C" B_DB CATS_IMP_EXP *backend_instantiate(JCR *jcr,
                                                   const char *db_socket,
                                                   bool mult_db_connections,
                                                   bool disable_batch_insert,
+                                                  bool try_reconnect,
+                                                  bool exit_on_fatal,
                                                   bool need_private)
 #else
 B_DB *db_init_database(JCR *jcr,
@@ -675,6 +679,8 @@ B_DB *db_init_database(JCR *jcr,
                        const char *db_socket,
                        bool mult_db_connections,
                        bool disable_batch_insert,
+                       bool try_reconnect,
+                       bool exit_on_fatal,
                        bool need_private)
 #endif
 {
@@ -709,6 +715,8 @@ B_DB *db_init_database(JCR *jcr,
                          db_socket,
                          mult_db_connections,
                          disable_batch_insert,
+                         try_reconnect,
+                         exit_on_fatal,
                          need_private));
 
 bail_out:
