@@ -3,7 +3,7 @@
 
    Copyright (C) 2000-2010 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -56,6 +56,8 @@ static char bandwidthcmd[] =
    "setbandwidth=%lld Job=%s\n";
 static char pluginoptionscmd[] =
    "pluginoptions %s\n";
+static char getSecureEraseCmd[] =
+   "getSecureEraseCmd\n";
 
 /* Responses received from File daemon */
 static char OKinc[] =
@@ -74,6 +76,8 @@ static char OKBandwidth[] =
    "2000 OK Bandwidth\n";
 static char OKPluginOptions[] =
    "2000 OK PluginOptions\n";
+static char OKgetSecureEraseCmd[] =
+   "2000 OK FDSecureEraseCmd %s\n";
 
 /* Forward referenced functions */
 static bool send_list_item(JCR *jcr, const char *code, char *item, BSOCK *fd);
@@ -84,99 +88,156 @@ extern DIRRES *director;
 #define INC_LIST 0
 #define EXC_LIST 1
 
-/*
- * Open connection with File daemon.
- * Try connecting every retry_interval (default 10 sec), and
- *   give up after max_retry_time (default 30 mins).
- */
-
-int connect_to_file_daemon(JCR *jcr, int retry_interval, int max_retry_time, bool verbose, bool start_job)
+static inline utime_t get_heartbeat_interval(CLIENTRES *res)
 {
-   BSOCK *fd;
-   char ed1[30];
+   utime_t heartbeat;
+
+   if (res->heartbeat_interval) {
+      heartbeat = res->heartbeat_interval;
+   } else {
+      heartbeat = me->heartbeat_interval;
+   }
+
+   return heartbeat;
+}
+
+/*
+ * Open connection to File daemon.
+ *
+ * Try connecting every retry_interval (default 10 sec), and
+ * give up after max_retry_time (default 30 mins).
+ */
+static inline bool connect_outbound_to_file_daemon(JCR *jcr, int retry_interval,
+                                                   int max_retry_time, bool verbose)
+{
+   bool result = false;
+   BSOCK *fd = NULL;
    utime_t heart_beat;
 
+   if (!is_connecting_to_client_allowed(jcr)) {
+      Dmsg1(120, "connecting to client \"%s\" is not allowed.\n", jcr->res.client->name());
+      return false;
+   }
+
    fd = New(BSOCK_TCP);
    if (me->nokeepalive) {
       fd->clear_keepalive();
    }
-   if (jcr->res.client->heartbeat_interval) {
-      heart_beat = jcr->res.client->heartbeat_interval;
+   heart_beat = get_heartbeat_interval(jcr->res.client);
+
+   char name[MAX_NAME_LENGTH + 100];
+   bstrncpy(name, _("Client: "), sizeof(name));
+   bstrncat(name, jcr->res.client->name(), sizeof(name));
+
+   fd->set_source_address(me->DIRsrc_addr);
+   if (!fd->connect(jcr,retry_interval,max_retry_time,
+                    heart_beat, name,
+                    jcr->res.client->address, NULL,
+                    jcr->res.client->FDport, verbose)) {
+      delete fd;
+      fd = NULL;
+      jcr->setJobStatus(JS_ErrorTerminated);
    } else {
-      heart_beat = me->heartbeat_interval;
+      jcr->file_bsock = fd;
+      jcr->authenticated = false;
+      Dmsg0(10, "Opened connection with File daemon\n");
+      result = true;
    }
 
+   return result;
+}
+
+bool connect_to_file_daemon(JCR *jcr, int retry_interval, int max_retry_time, bool verbose)
+{
+   bool result = false;
+
+   /*
+    * connection already exists.
+    * TODO: when is this the case? Is it really used? Does it than really need authentication?
+    */
    if (!jcr->file_bsock) {
-      char name[MAX_NAME_LENGTH + 100];
-      bstrncpy(name, _("Client: "), sizeof(name));
-      bstrncat(name, jcr->res.client->name(), sizeof(name));
-
-      fd->set_source_address(me->DIRsrc_addr);
-      if (!fd->connect(jcr,retry_interval,max_retry_time, heart_beat, name,
-                       jcr->res.client->address, NULL,
-                       jcr->res.client->FDport, verbose)) {
-         delete fd;
-         fd = NULL;
+      /*
+       * check without waiting, if waiting client connection exists.
+       */
+      if (!use_waiting_client(jcr, 0)) {
+         /*
+          * open connection to client
+          */
+         if (!connect_outbound_to_file_daemon(jcr, retry_interval, max_retry_time, verbose)) {
+            /*
+             * Check if a waiting client connection exist.
+             * If yes, use it, otherwise jcr->file_bsock will not be set.
+             */
+            use_waiting_client(jcr, max_retry_time);
+         }
       }
+   }
 
-      if (fd == NULL) {
+   /*
+    * connection have been established
+    */
+   if (jcr->file_bsock) {
+      jcr->setJobStatus(JS_Running);
+      if (authenticate_with_file_daemon(jcr)) {
+         result = true;
+      }
+   } else {
+      Jmsg(jcr, M_FATAL, 0, "Failed to connect to client \"%s\".\n", jcr->res.client->name());
+   }
+
+   if (!result) {
+     jcr->setJobStatus(JS_ErrorTerminated);
+   }
+
+   return result;
+}
+
+int send_job_info(JCR *jcr)
+{
+   BSOCK *fd = jcr->file_bsock;
+   char ed1[30];
+
+   /*
+    * Now send JobId and authorization key
+    */
+   if (jcr->sd_auth_key == NULL) {
+      jcr->sd_auth_key = bstrdup("dummy");
+   }
+
+   fd->fsend(jobcmd, edit_int64(jcr->JobId, ed1), jcr->Job, jcr->VolSessionId,
+             jcr->VolSessionTime, jcr->sd_auth_key);
+
+   if (!jcr->keep_sd_auth_key && !bstrcmp(jcr->sd_auth_key, "dummy")) {
+      memset(jcr->sd_auth_key, 0, strlen(jcr->sd_auth_key));
+   }
+
+   Dmsg1(100, ">filed: %s", fd->msg);
+   if (bget_dirmsg(fd) > 0) {
+      Dmsg1(110, "<filed: %s", fd->msg);
+      if (!bstrncmp(fd->msg, OKjob, strlen(OKjob))) {
+         Jmsg(jcr, M_FATAL, 0, _("File daemon \"%s\" rejected Job command: %s\n"),
+              jcr->res.client->hdr.name, fd->msg);
          jcr->setJobStatus(JS_ErrorTerminated);
          return 0;
-      }
-      Dmsg0(10, "Opened connection with File daemon\n");
-   } else {
-      fd = jcr->file_bsock;           /* use existing connection */
-   }
-   fd->res = (RES *)jcr->res.client;  /* save resource in BSOCK */
-   jcr->file_bsock = fd;
-   jcr->setJobStatus(JS_Running);
+      } else if (jcr->db) {
+         CLIENT_DBR cr;
 
-   if (!authenticate_file_daemon(jcr)) {
+         memset(&cr, 0, sizeof(cr));
+         bstrncpy(cr.Name, jcr->res.client->hdr.name, sizeof(cr.Name));
+         cr.AutoPrune = jcr->res.client->AutoPrune;
+         cr.FileRetention = jcr->res.client->FileRetention;
+         cr.JobRetention = jcr->res.client->JobRetention;
+         bstrncpy(cr.Uname, fd->msg+strlen(OKjob)+1, sizeof(cr.Uname));
+         if (!db_update_client_record(jcr, jcr->db, &cr)) {
+            Jmsg(jcr, M_WARNING, 0, _("Error updating Client record. ERR=%s\n"), db_strerror(jcr->db));
+         }
+      }
+   } else {
+      Jmsg(jcr, M_FATAL, 0, _("FD gave bad response to JobId command: %s\n"), bnet_strerror(fd));
       jcr->setJobStatus(JS_ErrorTerminated);
       return 0;
    }
 
-   if (start_job) {
-      /*
-       * Now send JobId and authorization key
-       */
-      if (jcr->sd_auth_key == NULL) {
-         jcr->sd_auth_key = bstrdup("dummy");
-      }
-      fd->fsend(jobcmd, edit_int64(jcr->JobId, ed1), jcr->Job, jcr->VolSessionId,
-                jcr->VolSessionTime, jcr->sd_auth_key);
-      if (!jcr->keep_sd_auth_key && !bstrcmp(jcr->sd_auth_key, "dummy")) {
-         memset(jcr->sd_auth_key, 0, strlen(jcr->sd_auth_key));
-      }
-      Dmsg1(100, ">filed: %s", fd->msg);
-      if (bget_dirmsg(fd) > 0) {
-          Dmsg1(110, "<filed: %s", fd->msg);
-          if (!bstrncmp(fd->msg, OKjob, strlen(OKjob))) {
-             Jmsg(jcr, M_FATAL, 0, _("File daemon \"%s\" rejected Job command: %s\n"),
-                jcr->res.client->hdr.name, fd->msg);
-             jcr->setJobStatus(JS_ErrorTerminated);
-             return 0;
-          } else if (jcr->db) {
-             CLIENT_DBR cr;
-             memset(&cr, 0, sizeof(cr));
-             bstrncpy(cr.Name, jcr->res.client->hdr.name, sizeof(cr.Name));
-             cr.AutoPrune = jcr->res.client->AutoPrune;
-             cr.FileRetention = jcr->res.client->FileRetention;
-             cr.JobRetention = jcr->res.client->JobRetention;
-             bstrncpy(cr.Uname, fd->msg+strlen(OKjob)+1, sizeof(cr.Uname));
-             if (!db_update_client_record(jcr, jcr->db, &cr)) {
-                Jmsg(jcr, M_WARNING, 0, _("Error updating Client record. ERR=%s\n"),
-                   db_strerror(jcr->db));
-             }
-          }
-      } else {
-         Jmsg(jcr, M_FATAL, 0, _("FD gave bad response to JobId command: %s\n"),
-         bnet_strerror(fd));
-         jcr->setJobStatus(JS_ErrorTerminated);
-         return 0;
-      }
-   }
-
    return 1;
 }
 
@@ -216,6 +277,35 @@ bool send_bwlimit_to_fd(JCR *jcr, const char *Job)
    return true;
 }
 
+bool send_secure_erase_req_to_fd(JCR *jcr)
+{
+   int32_t n;
+   BSOCK *fd = jcr->file_bsock;
+
+   if (!jcr->FDSecureEraseCmd) {
+      jcr->FDSecureEraseCmd = get_pool_memory(PM_NAME);
+   }
+
+   if (jcr->FDVersion > FD_VERSION_53) {
+      fd->fsend(getSecureEraseCmd);
+      while ((n = bget_dirmsg(fd)) >= 0) {
+         jcr->FDSecureEraseCmd = check_pool_memory_size(jcr->FDSecureEraseCmd, fd->msglen);
+         if (sscanf(fd->msg, OKgetSecureEraseCmd, jcr->FDSecureEraseCmd) == 1) {
+            Dmsg1(400, "Got FD Secure Erase Cmd: %s\n", jcr->FDSecureEraseCmd);
+            break;
+         } else {
+            Jmsg(jcr, M_WARNING, 0, _("Unexpected Client Secure Erase Cmd: %s\n"), fd->msg);
+            pm_strcpy(jcr->FDSecureEraseCmd, "*None*");
+            return false;
+         }
+      }
+   } else {
+      pm_strcpy(jcr->FDSecureEraseCmd, "*None*");
+   }
+
+   return true;
+}
+
 static inline void send_since_time(JCR *jcr)
 {
    char ed1[50];
@@ -597,20 +687,20 @@ int send_runscripts_commands(JCR *jcr)
       return 1;
    }
 
-   Dmsg0(120, "bdird: sending runscripts to fd\n");
+   Dmsg0(120, "dird: sending runscripts to fd\n");
 
    msg = get_pool_memory(PM_FNAME);
    ehost = get_pool_memory(PM_FNAME);
    foreach_alist(cmd, jcr->res.job->RunScripts) {
       if (cmd->can_run_at_level(jcr->getJobLevel()) && cmd->target) {
          ehost = edit_job_codes(jcr, ehost, cmd->target, "");
-         Dmsg2(200, "bdird: runscript %s -> %s\n", cmd->target, ehost);
+         Dmsg2(200, "dird: runscript %s -> %s\n", cmd->target, ehost);
 
          if (bstrcmp(ehost, jcr->res.client->name())) {
             pm_strcpy(msg, cmd->command);
             bash_spaces(msg);
 
-            Dmsg1(120, "bdird: sending runscripts to fd '%s'\n", cmd->command);
+            Dmsg1(120, "dird: sending runscripts to fd '%s'\n", cmd->command);
 
             fd->fsend(runscriptcmd,
                       cmd->on_success,
@@ -628,7 +718,7 @@ int send_runscripts_commands(JCR *jcr)
          /* TODO : we have to play with other client */
          /*
            else {
-           send command to an other client
+           send command to another client
            }
          */
       }
@@ -716,7 +806,7 @@ static int restore_object_handler(void *ctx, int num_fields, char **row)
                       jcr->db,
                       row[8],                /* Object  */
                       str_to_uint64(row[1]), /* Object length */
-                      &fd->msg,
+                      fd->msg,
                       &fd->msglen);
    fd->send();                           /* send object */
    octx->count++;
@@ -842,22 +932,28 @@ int get_attributes_and_put_in_catalog(JCR *jcr)
    BSOCK *fd;
    int n = 0;
    ATTR_DBR *ar = NULL;
-   POOL_MEM digest(PM_FNAME);
+   POOL_MEM digest(PM_MESSAGE);
 
    fd = jcr->file_bsock;
    jcr->jr.FirstIndex = 1;
    jcr->FileIndex = 0;
-   /* Start transaction allocates jcr->attr and jcr->ar if needed */
+
+   /*
+    * Start transaction allocates jcr->attr and jcr->ar if needed
+    */
    db_start_transaction(jcr, jcr->db);     /* start transaction if not already open */
    ar = jcr->ar;
 
-   Dmsg0(120, "bdird: waiting to receive file attributes\n");
-   /* Pickup file attributes and digest */
+   Dmsg0(120, "dird: waiting to receive file attributes\n");
+
+   /*
+    * Pickup file attributes and digest
+    */
    while (!fd->errors && (n = bget_dirmsg(fd)) > 0) {
       uint32_t file_index;
       int stream, len;
       char *p, *fn;
-      POOL_MEM Digest(PM_NAME);    /* either Verify opts or MD5/SHA1 digest */
+      POOL_MEM Digest(PM_MESSAGE);    /* Either Verify opts or MD5/SHA1 digest */
 
       if ((len = sscanf(fd->msg, "%ld %d %s", &file_index, &stream, Digest.c_str())) != 3) {
          Jmsg(jcr, M_FATAL, 0, _("<filed: bad attributes, expected 3 fields got %d\n"
@@ -866,7 +962,10 @@ int get_attributes_and_put_in_catalog(JCR *jcr)
          return 0;
       }
       p = fd->msg;
-      /* The following three fields were sscanf'ed above so skip them */
+
+      /*
+       * The following three fields were sscanf'ed above so skip them
+       */
       skip_nonspaces(&p);             /* skip FileIndex */
       skip_spaces(&p);
       skip_nonspaces(&p);             /* skip Stream */
@@ -882,7 +981,10 @@ int get_attributes_and_put_in_catalog(JCR *jcr)
                Jmsg1(jcr, M_FATAL, 0, _("Attribute create error. %s"), db_strerror(jcr->db));
             }
          }
-         /* Any cached attr is flushed so we can reuse jcr->attr and jcr->ar */
+
+         /*
+          * Any cached attr is flushed so we can reuse jcr->attr and jcr->ar
+          */
          fn = jcr->fname = check_pool_memory_size(jcr->fname, fd->msglen);
          while (*p != 0) {
             *fn++ = *p++;                /* copy filename */
@@ -908,32 +1010,39 @@ int get_attributes_and_put_in_catalog(JCR *jcr)
          Dmsg2(dbglvl, "dird<filed: stream=%d %s\n", stream, jcr->fname);
          Dmsg1(dbglvl, "dird<filed: attr=%s\n", ar->attr);
          jcr->FileId = ar->FileId;
-      /*
-       * First, get STREAM_UNIX_ATTRIBUTES and fill ATTR_DBR structure
-       * Next, we CAN have a CRYPTO_DIGEST, so we fill ATTR_DBR with it (or not)
-       * When we get a new STREAM_UNIX_ATTRIBUTES, we known that we can add file to the catalog
-       * At the end, we have to add the last file
-       */
       } else if (crypto_digest_stream_type(stream) != CRYPTO_DIGEST_NONE) {
+         size_t length;
+
+         /*
+          * First, get STREAM_UNIX_ATTRIBUTES and fill ATTR_DBR structure
+          * Next, we CAN have a CRYPTO_DIGEST, so we fill ATTR_DBR with it (or not)
+          * When we get a new STREAM_UNIX_ATTRIBUTES, we known that we can add file to the catalog
+          * At the end, we have to add the last file
+          */
          if (jcr->FileIndex != (uint32_t)file_index) {
             Jmsg3(jcr, M_ERROR, 0, _("%s index %d not same as attributes %d\n"),
                stream_to_ascii(stream), file_index, jcr->FileIndex);
             continue;
          }
+
          ar->Digest = digest.c_str();
          ar->DigestType = crypto_digest_stream_type(stream);
-         db_escape_string(jcr, jcr->db, digest.c_str(), Digest.c_str(), strlen(Digest.c_str()));
+         length = strlen(Digest.c_str());
+         digest.check_size(length * 2 + 1);
+         db_escape_string(jcr, jcr->db, digest.c_str(), Digest.c_str(), length);
          Dmsg4(dbglvl, "stream=%d DigestLen=%d Digest=%s type=%d\n", stream,
                strlen(digest.c_str()), digest.c_str(), ar->DigestType);
       }
       jcr->jr.JobFiles = jcr->JobFiles = file_index;
       jcr->jr.LastIndex = file_index;
    }
+
    if (is_bnet_error(fd)) {
       Jmsg1(jcr, M_FATAL, 0, _("<filed: Network error getting attributes. ERR=%s\n"),
             fd->bstrerror());
       return 0;
    }
+
    if (jcr->cached_attribute) {
       Dmsg3(dbglvl, "Cached attr with digest. Stream=%d fname=%s attr=%s\n", ar->Stream,
          ar->fname, ar->attr);
@@ -942,7 +1051,9 @@ int get_attributes_and_put_in_catalog(JCR *jcr)
       }
       jcr->cached_attribute = false;
    }
+
    jcr->setJobStatus(JS_Terminated);
+
    return 1;
 }
 
@@ -954,7 +1065,7 @@ bool cancel_file_daemon_job(UAContext *ua, JCR *jcr)
    BSOCK *fd;
 
    ua->jcr->res.client = jcr->res.client;
-   if (!connect_to_file_daemon(ua->jcr, 10, me->FDConnectTimeout, true, false)) {
+   if (!connect_to_file_daemon(ua->jcr, 10, me->FDConnectTimeout, true)) {
       ua->error_msg(_("Failed to connect to File daemon.\n"));
       return false;
    }
@@ -993,7 +1104,7 @@ void do_native_client_status(UAContext *ua, CLIENTRES *client, char *cmd)
                    client->name(), client->address, client->FDport);
    }
 
-   if (!connect_to_file_daemon(ua->jcr, 1, 15, false, false)) {
+   if (!connect_to_file_daemon(ua->jcr, 1, 15, false)) {
       ua->send_msg(_("Failed to connect to Client %s.\n====\n"),
          client->name());
       if (ua->jcr->file_bsock) {
@@ -1044,7 +1155,7 @@ void do_client_resolve(UAContext *ua, CLIENTRES *client)
                    client->name(), client->address, client->FDport);
    }
 
-   if (!connect_to_file_daemon(ua->jcr, 1, 15, false, false)) {
+   if (!connect_to_file_daemon(ua->jcr, 1, 15, false)) {
       ua->send_msg(_("Failed to connect to Client %s.\n====\n"),
          client->name());
       if (ua->jcr->file_bsock) {
@@ -1076,3 +1187,50 @@ void do_client_resolve(UAContext *ua, CLIENTRES *client)
 
    return;
 }
+
+/*
+ * After receiving a connection (in socket_server.c) if it is
+ * from the File daemon, this routine is called.
+ */
+void *handle_filed_connection(CONNECTION_POOL *connections, BSOCK *fd,
+                              char *client_name, int fd_protocol_version)
+{
+   CLIENTRES *client_resource;
+   CONNECTION *connection = NULL;
+
+   client_resource = (CLIENTRES *)GetResWithName(R_CLIENT, client_name);
+   if (!client_resource) {
+      Emsg1(M_WARNING, 0, "Client \"%s\" tries to connect, "
+                          "but no matching resource is defined.\n", client_name);
+      goto getout;
+   }
+
+   if (!is_connect_from_client_allowed(client_resource)) {
+      Emsg1(M_WARNING, 0, "Client \"%s\" tries to connect, "
+                          "but does not have the required permission.\n", client_name);
+      goto getout;
+   }
+
+   if (!authenticate_file_daemon(fd, client_name)) {
+      goto getout;
+   }
+
+   Dmsg1(20, "Connected to file daemon %s\n", client_name);
+
+   connection = connections->add_connection(client_name, fd_protocol_version, fd, true);
+   if (!connection) {
+      Emsg0(M_ERROR, 0, "Failed to add connection to pool.\n");
+      goto getout;
+   }
+
+   /*
+    * The connection is now kept in connection_pool.
+    * This thread is no longer required and will end now.
+    */
+   return NULL;
+
+getout:
+   fd->close();
+   delete(fd);
+   return NULL;
+}
