@@ -3,7 +3,7 @@
 
    Copyright (C) 2003-2011 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -41,22 +41,27 @@
 #include "bareos.h"
 #include "dird.h"
 
-extern JCR *jobs;
+static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 
 /* Forward referenced functions */
 extern "C" void *jobq_server(void *arg);
 extern "C" void *sched_wait(void *arg);
 
-static int  start_server(jobq_t *jq);
+static int start_server(jobq_t *jq);
 static bool acquire_resources(JCR *jcr);
 static bool reschedule_job(JCR *jcr, jobq_t *jq, jobq_item_t *je);
+static bool inc_client_concurrency(JCR *jcr);
+static void dec_client_concurrency(JCR *jcr);
+static bool inc_job_concurrency(JCR *jcr);
+static void dec_job_concurrency(JCR *jcr);
+static bool inc_write_store(JCR *jcr);
 static void dec_write_store(JCR *jcr);
 
 /*
  * Initialize a job queue
  *
- *  Returns: 0 on success
- *           errno on failure
+ * Returns: 0 on success
+ *          errno on failure
  */
 int jobq_init(jobq_t *jq, int threads, void *(*engine)(void *arg))
 {
@@ -91,10 +96,14 @@ int jobq_init(jobq_t *jq, int threads, void *(*engine)(void *arg))
    jq->idle_workers = 0;              /* no idle threads */
    jq->engine = engine;               /* routine to run */
    jq->valid = JOBQ_VALID;
-   /* Initialize the job queues */
+
+   /*
+    * Initialize the job queues
+    */
    jq->waiting_jobs = New(dlist(item, &item->link));
    jq->running_jobs = New(dlist(item, &item->link));
    jq->ready_jobs = New(dlist(item, &item->link));
+
    return 0;
 }
 
@@ -153,13 +162,12 @@ struct wait_pkt {
 
 /*
  * Wait until schedule time arrives before starting. Normally
- *  this routine is only used for jobs started from the console
- *  for which the user explicitly specified a start time. Otherwise
- *  most jobs are put into the job queue only when their
- *  scheduled time arives.
+ * this routine is only used for jobs started from the console
+ * for which the user explicitly specified a start time. Otherwise
+ * most jobs are put into the job queue only when their
+ * scheduled time arives.
  */
-extern "C"
-void *sched_wait(void *arg)
+extern "C" void *sched_wait(void *arg)
 {
    JCR *jcr = ((wait_pkt *)arg)->jcr;
    jobq_t *jq = ((wait_pkt *)arg)->jq;
@@ -169,12 +177,18 @@ void *sched_wait(void *arg)
    free(arg);
    time_t wtime = jcr->sched_time - time(NULL);
    jcr->setJobStatus(JS_WaitStartTime);
-   /* Wait until scheduled time arrives */
+
+   /*
+    * Wait until scheduled time arrives
+    */
    if (wtime > 0) {
       Jmsg(jcr, M_INFO, 0, _("Job %s waiting %d seconds for scheduled start time.\n"),
          jcr->Job, wtime);
    }
-   /* Check every 30 seconds if canceled */
+
+   /*
+    * Check every 30 seconds if canceled
+    */
    while (wtime > 0) {
       Dmsg3(2300, "Waiting on sched time, jobid=%d secs=%d use=%d\n",
          jcr->JobId, wtime, jcr->use_count());
@@ -191,12 +205,13 @@ void *sched_wait(void *arg)
    jobq_add(jq, jcr);
    free_jcr(jcr);                     /* we are done with jcr */
    Dmsg0(2300, "Exit sched_wait\n");
+
    return NULL;
 }
 
 /*
- *  Add a job to the queue
- *    jq is a queue that was created with jobq_init
+ * Add a job to the queue
+ * jq is a queue that was created with jobq_init
  */
 int jobq_add(jobq_t *jq, JCR *jcr)
 {
@@ -208,7 +223,9 @@ int jobq_add(jobq_t *jq, JCR *jcr)
    wait_pkt *sched_pkt;
 
    if (!jcr->term_wait_inited) {
-      /* Initialize termination condition variable */
+      /*
+       * Initialize termination condition variable
+       */
       if ((status = pthread_cond_init(&jcr->term_wait, NULL)) != 0) {
          berrno be;
          Jmsg1(jcr, M_FATAL, 0, _("Unable to init job cond variable: ERR=%s\n"), be.bstrerror(status));
@@ -246,14 +263,20 @@ int jobq_add(jobq_t *jq, JCR *jcr)
    }
    item->jcr = jcr;
 
-   /* While waiting in a queue this job is not attached to a thread */
+   /*
+    * While waiting in a queue this job is not attached to a thread
+    */
    set_jcr_in_tsd(INVALID_JCR);
    if (job_canceled(jcr)) {
-      /* Add job to ready queue so that it is canceled quickly */
+      /*
+       * Add job to ready queue so that it is canceled quickly
+       */
       jq->ready_jobs->prepend(item);
       Dmsg1(2300, "Prepended job=%d to ready queue\n", jcr->JobId);
    } else {
-      /* Add this job to the wait queue in priority sorted order */
+      /*
+       * Add this job to the wait queue in priority sorted order
+       */
       foreach_dlist(li, jq->waiting_jobs) {
          Dmsg2(2300, "waiting item jobid=%d priority=%d\n",
             li->jcr->JobId, li->jcr->JobPriority);
@@ -265,14 +288,19 @@ int jobq_add(jobq_t *jq, JCR *jcr)
             break;
          }
       }
-      /* If not jobs in wait queue, append it */
+
+      /*
+       * If not jobs in wait queue, append it
+       */
       if (!inserted) {
          jq->waiting_jobs->append(item);
          Dmsg1(2300, "Appended item jobid=%d to waiting queue\n", jcr->JobId);
       }
    }
 
-   /* Ensure that at least one server looks at the queue. */
+   /*
+    * Ensure that at least one server looks at the queue.
+    */
    status = start_server(jq);
 
    V(jq->mutex);
@@ -281,13 +309,13 @@ int jobq_add(jobq_t *jq, JCR *jcr)
 }
 
 /*
- *  Remove a job from the job queue. Used only by cancel_job().
- *    jq is a queue that was created with jobq_init
- *    work_item is an element of work
+ * Remove a job from the job queue. Used only by cancel_job().
+ * jq is a queue that was created with jobq_init
+ * work_item is an element of work
  *
- *   Note, it is "removed" from the job queue.
- *    If you want to cancel it, you need to provide some external means
- *    of doing so (e.g. pthread_kill()).
+ * Note, it is "removed" from the job queue.
+ * If you want to cancel it, you need to provide some external means
+ * of doing so (e.g. pthread_kill()).
  */
 int jobq_remove(jobq_t *jq, JCR *jcr)
 {
@@ -313,7 +341,9 @@ int jobq_remove(jobq_t *jq, JCR *jcr)
       return EINVAL;
    }
 
-   /* Move item to be the first on the list */
+   /*
+    * Move item to be the first on the list
+    */
    jq->waiting_jobs->remove(item);
    jq->ready_jobs->prepend(item);
    Dmsg2(2300, "jobq_remove jobid=%d jcr=0x%x moved to ready queue\n", jcr->JobId, jcr);
@@ -325,7 +355,6 @@ int jobq_remove(jobq_t *jq, JCR *jcr)
    return status;
 }
 
-
 /*
  * Start the server thread if it isn't already running
  */
@@ -335,9 +364,10 @@ static int start_server(jobq_t *jq)
    pthread_t id;
 
    /*
-    * if any threads are idle, wake one.
-    *   Actually we do a broadcast because on /lib/tls
-    *   these signals seem to get lost from time to time.
+    * If any threads are idle, wake one.
+    *
+    * Actually we do a broadcast because on /lib/tls
+    * these signals seem to get lost from time to time.
     */
    if (jq->idle_workers > 0) {
       Dmsg0(2300, "Signal worker to wake up\n");
@@ -348,7 +378,9 @@ static int start_server(jobq_t *jq)
       }
    } else if (jq->num_workers < jq->max_workers) {
       Dmsg0(2300, "Create worker thread\n");
-      /* No idle threads so create a new one */
+      /*
+       * No idle threads so create a new one
+       */
       set_thread_concurrency(jq->max_workers + 1);
       jq->num_workers++;
       if ((status = pthread_create(&id, &jq->attr, jobq_server, (void *)jq)) != 0) {
@@ -364,7 +396,7 @@ static int start_server(jobq_t *jq)
 /*
  * This is the worker thread that serves the job queue.
  * When all the resources are acquired for the job,
- *  it will call the user's engine.
+ * it will call the user's engine.
  */
 extern "C" void *jobq_server(void *arg)
 {
@@ -411,12 +443,14 @@ extern "C" void *jobq_server(void *arg)
             break;
          }
       }
+
       /*
        * If anything is in the ready queue, run it
        */
       Dmsg0(2300, "Checking ready queue.\n");
       while (!jq->ready_jobs->empty() && !jq->quit) {
          JCR *jcr;
+
          je = (jobq_item_t *)jq->ready_jobs->first();
          jcr = je->jcr;
          jq->ready_jobs->remove(je);
@@ -467,27 +501,13 @@ extern "C" void *jobq_server(void *arg)
 
          /*
           * Release locks if acquired. Note, they will not have
-          * been acquired for jobs canceled before they were
-          * put into the ready queue.
+          * been acquired for jobs canceled before they were put into the ready queue.
           */
          if (jcr->acquired_resource_locks) {
             dec_read_store(jcr);
             dec_write_store(jcr);
-            if (jcr->res.client) {
-               /*
-                * Some Job Types are excluded from the client concurrency as they have no
-                * interaction with the client at all.
-                */
-               switch (jcr->getJobType()) {
-               case JT_MIGRATE:
-               case JT_COPY:
-                  break;
-               default:
-                  jcr->res.client->NumConcurrentJobs--;
-                  break;
-               }
-            }
-            jcr->res.job->NumConcurrentJobs--;
+            dec_client_concurrency(jcr);
+            dec_job_concurrency(jcr);
             jcr->acquired_resource_locks = false;
          }
 
@@ -555,9 +575,10 @@ extern "C" void *jobq_server(void *arg)
             /*
              * Take only jobs of correct Priority
              */
-            if (!(jcr->JobPriority == Priority
-                  || (jcr->JobPriority < Priority &&
-                      jcr->res.job->allow_mixed_priority && running_allow_mix))) {
+            if (!(jcr->JobPriority == Priority ||
+                  (jcr->JobPriority < Priority &&
+                   jcr->res.job->allow_mixed_priority &&
+                   running_allow_mix))) {
                jcr->setJobStatus(JS_WaitPriority);
                break;
             }
@@ -694,7 +715,6 @@ static bool reschedule_job(JCR *jcr, jobq_t *jq, jobq_item_t *je)
            jcr->Job, dt, (int)jcr->res.job->RescheduleInterval, dt2);
       dird_free_jcr_pointers(jcr);     /* partial cleanup old stuff */
       jcr->JobStatus = -1;
-      jcr->setJobStatus(JS_WaitStartTime);
       jcr->SDJobStatus = 0;
       jcr->JobErrors = 0;
       if (!allow_duplicate_job(jcr)) {
@@ -705,6 +725,7 @@ static bool reschedule_job(JCR *jcr, jobq_t *jq, jobq_item_t *je)
        * Only jobs with no output or Incomplete jobs can run on same JCR
        */
       if (jcr->JobBytes == 0) {
+         update_job_end(jcr, JS_WaitStartTime);
          Dmsg2(2300, "Requeue job=%d use=%d\n", jcr->JobId, jcr->use_count());
          V(jq->mutex);
          jcr->jr.RealEndTime = 0;
@@ -722,6 +743,7 @@ static bool reschedule_job(JCR *jcr, jobq_t *jq, jobq_item_t *je)
           * conflicts.  We now create a new job, copying the
           * appropriate fields.
           */
+         jcr->setJobStatus(JS_WaitStartTime);
          njcr = new_jcr(sizeof(JCR), dird_free_jcr);
          set_jcr_defaults(njcr, jcr->res.job);
          njcr->reschedule_count = jcr->reschedule_count;
@@ -742,12 +764,12 @@ static bool reschedule_job(JCR *jcr, jobq_t *jq, jobq_item_t *je)
          njcr->JobStatus = -1;
          njcr->setJobStatus(jcr->JobStatus);
          if (jcr->res.rstore) {
-            copy_rstorage(njcr, jcr->rstorage, _("previous Job"));
+            copy_rstorage(njcr, jcr->res.rstorage, _("previous Job"));
          } else {
             free_rstorage(njcr);
          }
          if (jcr->res.wstore) {
-            copy_wstorage(njcr, jcr->wstorage, _("previous Job"));
+            copy_wstorage(njcr, jcr->res.wstorage, _("previous Job"));
          } else {
             free_wstorage(njcr);
          }
@@ -773,30 +795,59 @@ static bool reschedule_job(JCR *jcr, jobq_t *jq, jobq_item_t *je)
  */
 static bool acquire_resources(JCR *jcr)
 {
-   bool skip_this_jcr = false;
-
+   /*
+    * Set that we didn't acquire any resourse locks yet.
+    */
    jcr->acquired_resource_locks = false;
-/*
- * Turning this code off is likely to cause some deadlocks,
- *   but we do not really have enough information here to
- *   know if this is really a deadlock (it may be a dual drive
- *   autochanger), and in principle, the SD reservation system
- *   should detect these deadlocks, so push the work off on it.
- */
+
+   /*
+    * Some Job Types are excluded from the client and storage concurrency
+    * as they have no interaction with the client or storage at all.
+    */
+   switch (jcr->getJobType()) {
+   case JT_MIGRATE:
+   case JT_COPY:
+   case JT_CONSOLIDATE:
+      /*
+       * Migration/Copy and Consolidation jobs are not counted for client concurrency
+       * as they do not touch the client at all
+       */
+      jcr->IgnoreClientConcurrency = true;
+      Dmsg1(200, "Skipping migrate/copy Job %s for client concurrency\n", jcr->Job);
+
+      if (jcr->MigrateJobId == 0) {
+         /*
+          * Migration/Copy control jobs are not counted for storage concurrency
+          * as they do not touch the storage at all
+          */
+         Dmsg1(200, "Skipping migrate/copy Control Job %s for storage concurrency\n", jcr->Job);
+         jcr->IgnoreStorageConcurrency = true;
+      }
+      break;
+   default:
+      break;
+   }
+
 #ifdef xxx
+   /*
+    * Turning this code off is likely to cause some deadlocks,
+    * but we do not really have enough information here to
+    * know if this is really a deadlock (it may be a dual drive
+    * autochanger), and in principle, the SD reservation system
+    * should detect these deadlocks, so push the work off on it.
+    */
    if (jcr->res.rstore && jcr->res.rstore == jcr->res.wstore) { /* possible deadlock */
       Jmsg(jcr, M_FATAL, 0, _("Job canceled. Attempt to read and write same device.\n"
-         "    Read storage \"%s\" (From %s) -- Write storage \"%s\" (From %s)\n"),
-         jcr->res.rstore->name(), jcr->res.rstore_source, jcr->res.wstore->name(), jcr->res.wstore_source);
+           "    Read storage \"%s\" (From %s) -- Write storage \"%s\" (From %s)\n"),
+           jcr->res.rstore->name(), jcr->res.rstore_source, jcr->res.wstore->name(), jcr->res.wstore_source);
       jcr->setJobStatus(JS_Canceled);
 
       return false;
    }
 #endif
+
    if (jcr->res.rstore) {
-      Dmsg1(200, "Rstore=%s\n", jcr->res.rstore->name());
       if (!inc_read_store(jcr)) {
-         Dmsg1(200, "Fail rncj=%d\n", jcr->res.rstore->NumConcurrentJobs);
          jcr->setJobStatus(JS_WaitStoreRes);
 
          return false;
@@ -804,75 +855,32 @@ static bool acquire_resources(JCR *jcr)
    }
 
    if (jcr->res.wstore) {
-      Dmsg1(200, "Wstore=%s\n", jcr->res.wstore->name());
-      if (jcr->res.wstore->NumConcurrentJobs < jcr->res.wstore->MaxConcurrentJobs) {
-         jcr->res.wstore->NumConcurrentJobs++;
-         Dmsg1(200, "Inc wncj=%d\n", jcr->res.wstore->NumConcurrentJobs);
-      } else if (jcr->res.rstore) {
+      if (!inc_write_store(jcr)) {
          dec_read_store(jcr);
-         skip_this_jcr = true;
-      } else {
-         Dmsg1(200, "Fail wncj=%d\n", jcr->res.wstore->NumConcurrentJobs);
-         skip_this_jcr = true;
+         jcr->setJobStatus(JS_WaitStoreRes);
+
+         return false;
       }
    }
 
-   if (skip_this_jcr) {
-      jcr->setJobStatus(JS_WaitStoreRes);
-
-      return false;
-   }
-
-   /*
-    * Some Job Types are excluded from the client concurrency as they have no
-    * interaction with the client at all.
-    */
-   switch (jcr->getJobType()) {
-   case JT_MIGRATE:
-   case JT_COPY:
-      break;
-   default:
-      if (jcr->res.client) {
-         if (jcr->res.client->NumConcurrentJobs < jcr->res.client->MaxConcurrentJobs) {
-            jcr->res.client->NumConcurrentJobs++;
-         } else {
-            /*
-             * Back out previous locks
-             */
-            dec_write_store(jcr);
-            dec_read_store(jcr);
-            jcr->setJobStatus(JS_WaitClientRes);
-
-            return false;
-         }
-      }
-      break;
-   }
-
-   if (jcr->res.job->NumConcurrentJobs < jcr->res.job->MaxConcurrentJobs) {
-      jcr->res.job->NumConcurrentJobs++;
-   } else {
+   if (!inc_client_concurrency(jcr)) {
       /*
        * Back out previous locks
        */
       dec_write_store(jcr);
       dec_read_store(jcr);
+      jcr->setJobStatus(JS_WaitClientRes);
 
+      return false;
+   }
+
+   if (!inc_job_concurrency(jcr)) {
       /*
-       * Some Job Types are excluded from the client concurrency as they have no
-       * interaction with the client at all.
+       * Back out previous locks
        */
-      switch (jcr->getJobType()) {
-      case JT_MIGRATE:
-      case JT_COPY:
-         break;
-      default:
-         if (jcr->res.client) {
-            jcr->res.client->NumConcurrentJobs--;
-         }
-         break;
-      }
-
+      dec_write_store(jcr);
+      dec_read_store(jcr);
+      dec_client_concurrency(jcr);
       jcr->setJobStatus(JS_WaitJobRes);
 
       return false;
@@ -883,44 +891,153 @@ static bool acquire_resources(JCR *jcr)
    return true;
 }
 
-static pthread_mutex_t rstore_mutex = PTHREAD_MUTEX_INITIALIZER;
+static bool inc_client_concurrency(JCR *jcr)
+{
+   if (!jcr->res.client || jcr->IgnoreClientConcurrency) {
+      return true;
+   }
+
+   P(mutex);
+   if (jcr->res.client->rcs->NumConcurrentJobs < jcr->res.client->MaxConcurrentJobs) {
+      jcr->res.client->rcs->NumConcurrentJobs++;
+      Dmsg2(50, "Inc Client=%s rncj=%d\n",
+            jcr->res.client->name(), jcr->res.client->rcs->NumConcurrentJobs);
+      V(mutex);
+
+      return true;
+   }
+
+   V(mutex);
+
+   return false;
+}
+
+static void dec_client_concurrency(JCR *jcr)
+{
+   if (jcr->IgnoreClientConcurrency) {
+      return;
+   }
+
+   P(mutex);
+   if (jcr->res.client) {
+      jcr->res.client->rcs->NumConcurrentJobs--;
+      Dmsg2(50, "Dec Client=%s rncj=%d\n",
+            jcr->res.client->name(), jcr->res.client->rcs->NumConcurrentJobs);
+   }
+   V(mutex);
+}
+
+static bool inc_job_concurrency(JCR *jcr)
+{
+   P(mutex);
+   if (jcr->res.job->rjs->NumConcurrentJobs < jcr->res.job->MaxConcurrentJobs) {
+      jcr->res.job->rjs->NumConcurrentJobs++;
+      Dmsg2(50, "Inc Job=%s rncj=%d\n",
+            jcr->res.job->name(), jcr->res.job->rjs->NumConcurrentJobs);
+      V(mutex);
+
+      return true;
+   }
+
+   V(mutex);
+
+   return false;
+}
+
+static void dec_job_concurrency(JCR *jcr)
+{
+   P(mutex);
+   jcr->res.job->rjs->NumConcurrentJobs--;
+   Dmsg2(50, "Dec Job=%s rncj=%d\n",
+         jcr->res.job->name(), jcr->res.job->rjs->NumConcurrentJobs);
+   V(mutex);
+}
 
 /*
  * Note: inc_read_store() and dec_read_store() are
- *   called from select_rstore() in src/dird/restore.c
+ * called from select_next_rstore() in src/dird/job.c
  */
 bool inc_read_store(JCR *jcr)
 {
-   P(rstore_mutex);
-   if (jcr->res.rstore->NumConcurrentJobs < jcr->res.rstore->MaxConcurrentJobs) {
-      jcr->res.rstore->NumConcurrentReadJobs++;
-      jcr->res.rstore->NumConcurrentJobs++;
-      Dmsg1(200, "Inc rncj=%d\n", jcr->res.rstore->NumConcurrentJobs);
-      V(rstore_mutex);
+   if (jcr->IgnoreStorageConcurrency) {
       return true;
    }
-   V(rstore_mutex);
+
+   P(mutex);
+   if (jcr->res.rstore->rss->NumConcurrentJobs < jcr->res.rstore->MaxConcurrentJobs) {
+      jcr->res.rstore->rss->NumConcurrentReadJobs++;
+      jcr->res.rstore->rss->NumConcurrentJobs++;
+      Dmsg2(50, "Inc Rstore=%s rncj=%d\n",
+            jcr->res.rstore->name(), jcr->res.rstore->rss->NumConcurrentJobs);
+      V(mutex);
+
+      return true;
+   }
+   V(mutex);
+
+   Dmsg2(50, "Fail to acquire Rstore=%s rncj=%d\n",
+         jcr->res.rstore->name(), jcr->res.rstore->rss->NumConcurrentJobs);
+
    return false;
 }
 
 void dec_read_store(JCR *jcr)
 {
-   if (jcr->res.rstore) {
-      P(rstore_mutex);
-      jcr->res.rstore->NumConcurrentReadJobs--;    /* back out rstore */
-      jcr->res.rstore->NumConcurrentJobs--;        /* back out rstore */
-      Dmsg1(200, "Dec rncj=%d\n", jcr->res.rstore->NumConcurrentJobs);
-      V(rstore_mutex);
-      ASSERT(jcr->res.rstore->NumConcurrentReadJobs >= 0);
-      ASSERT(jcr->res.rstore->NumConcurrentJobs >= 0);
+   if (jcr->res.rstore && !jcr->IgnoreStorageConcurrency) {
+      P(mutex);
+      jcr->res.rstore->rss->NumConcurrentReadJobs--;
+      jcr->res.rstore->rss->NumConcurrentJobs--;
+      Dmsg2(50, "Dec Rstore=%s rncj=%d\n",
+            jcr->res.rstore->name(), jcr->res.rstore->rss->NumConcurrentJobs);
+
+      if (jcr->res.rstore->rss->NumConcurrentReadJobs < 0) {
+         Jmsg(jcr, M_FATAL, 0, _("NumConcurrentReadJobs Dec Rstore=%s rncj=%d\n"),
+              jcr->res.rstore->name(), jcr->res.rstore->rss->NumConcurrentReadJobs);
+      }
+
+      if (jcr->res.rstore->rss->NumConcurrentJobs < 0) {
+         Jmsg(jcr, M_FATAL, 0, _("NumConcurrentJobs Dec Rstore=%s rncj=%d\n"),
+              jcr->res.rstore->name(), jcr->res.rstore->rss->NumConcurrentJobs);
+      }
+      V(mutex);
    }
 }
 
+static bool inc_write_store(JCR *jcr)
+{
+   if (jcr->IgnoreStorageConcurrency) {
+      return true;
+   }
+
+   P(mutex);
+   if (jcr->res.wstore->rss->NumConcurrentJobs < jcr->res.wstore->MaxConcurrentJobs) {
+      jcr->res.wstore->rss->NumConcurrentJobs++;
+      Dmsg2(50, "Inc Wstore=%s wncj=%d\n",
+            jcr->res.wstore->name(), jcr->res.wstore->rss->NumConcurrentJobs);
+      V(mutex);
+
+      return true;
+   }
+   V(mutex);
+
+   Dmsg2(50, "Fail to acquire Wstore=%s wncj=%d\n",
+         jcr->res.wstore->name(), jcr->res.wstore->rss->NumConcurrentJobs);
+
+   return false;
+}
+
 static void dec_write_store(JCR *jcr)
 {
-   if (jcr->res.wstore) {
-      jcr->res.wstore->NumConcurrentJobs--;
-      Dmsg1(200, "Dec wncj=%d\n", jcr->res.wstore->NumConcurrentJobs);
-      ASSERT(jcr->res.wstore->NumConcurrentJobs >= 0);
+   if (jcr->res.wstore && !jcr->IgnoreStorageConcurrency) {
+      P(mutex);
+      jcr->res.wstore->rss->NumConcurrentJobs--;
+      Dmsg2(50, "Dec Wstore=%s wncj=%d\n",
+            jcr->res.wstore->name(), jcr->res.wstore->rss->NumConcurrentJobs);
+
+      if (jcr->res.wstore->rss->NumConcurrentJobs < 0) {
+         Jmsg(jcr, M_FATAL, 0, _("NumConcurrentJobs Dec Wstore=%s wncj=%d\n"),
+              jcr->res.wstore->name(), jcr->res.wstore->rss->NumConcurrentJobs);
+      }
+      V(mutex);
    }
 }
