@@ -63,58 +63,6 @@ static int dbglvl = 100;
 #define AVG_NR_BYTES_PER_ENTRY 256
 #define B_PAGE_SIZE 4096
 
-extern "C" int bndmp_fhdb_lmdb_add_file(struct ndmlog *ixlog, int tagc, char *raw_name,
-                                        ndmp9_file_stat *fstat)
-{
-   NIS *nis = (NIS *)ixlog->ctx;
-
-   nis->jcr->lock();
-   nis->jcr->JobFiles++;
-   nis->jcr->unlock();
-
-   if (nis->save_filehist) {
-      int8_t FileType = 0;
-      POOL_MEM attribs(PM_FNAME),
-               pathname(PM_FNAME);
-
-      /*
-       * Every file entry is relative from the filesystem currently being backed up.
-       */
-      Dmsg2(100, "bndmp_fhdb_lmdb_add_file: New filename ==> %s/%s\n", nis->filesystem, raw_name);
-
-      if (nis->jcr->ar) {
-         /*
-          * See if this is the top level entry of the tree e.g. len == 0
-          */
-         if (strlen(raw_name) == 0) {
-            ndmp_convert_fstat(fstat, nis->FileIndex, &FileType, attribs);
-
-            pm_strcpy(pathname, nis->filesystem);
-            pm_strcat(pathname, "/");
-            return 0;
-         } else {
-            ndmp_convert_fstat(fstat, nis->FileIndex, &FileType, attribs);
-
-            pm_strcpy(pathname, nis->filesystem);
-            pm_strcat(pathname, "/");
-            pm_strcat(pathname, raw_name);
-
-            if (FileType == FT_DIREND) {
-               /*
-                * A directory needs to end with a slash.
-                */
-               pm_strcat(pathname, "/");
-            }
-         }
-
-         ndmp_store_attribute_record(nis->jcr, pathname.c_str(), nis->virtual_filename, attribs.c_str(), FileType,
-                                     0, (fstat->fh_info.valid == NDMP9_VALIDITY_VALID) ? fstat->fh_info.value : 0);
-      }
-   }
-
-   return 0;
-}
-
 extern "C" int bndmp_fhdb_lmdb_add_dir(struct ndmlog *ixlog, int tagc, char *raw_name,
                                        ndmp9_u_quad dir_node, ndmp9_u_quad node)
 {
@@ -159,7 +107,6 @@ retry:
       result = mdb_put(fhdb_state->db_rw_txn, fhdb_state->db_dbi, &key, &data, 0);
       switch (result) {
       case 0:
-         //Jmsg3(nis->jcr, M_INFO, 0,  "added file \"%s\", node=%llu, dir_node=%llu\n", raw_name, payload->node, payload->dir_node);
          Dmsg3(dbglvl, "added file \"%s\", node=%llu, dir_node=%llu\n", raw_name, payload->node, payload->dir_node);
          break;
       case MDB_TXN_FULL:
@@ -429,7 +376,7 @@ void ndmp_fhdb_lmdb_register(struct ndmlog *ixlog)
    /*
     * Register the FileHandleDB callbacks.
     */
-   fhdb_callbacks.add_file = bndmp_fhdb_lmdb_add_file;
+   fhdb_callbacks.add_file = bndmp_fhdb_add_file;
    fhdb_callbacks.add_dir = bndmp_fhdb_lmdb_add_dir;
    fhdb_callbacks.add_node = bndmp_fhdb_lmdb_add_node;
    fhdb_callbacks.add_dirnode_root = bndmp_fhdb_lmdb_add_dirnode_root;
@@ -667,8 +614,12 @@ static inline void process_lmdb(NIS *nis, struct fhdb_state *fhdb_state)
             if (FileType == FT_DIREND) {
                /*
                 * split_path_and_filename() expects directories to end with a '/'
+                * so append '/' if full_path does not already end with '/'
                 */
-               pm_strcat(full_path, "/");
+               if ( full_path.c_str()[strlen(full_path.c_str()) - 1] != '/' ) {
+                  Dmsg1(100, ("appending / to %s \n"), full_path.c_str());
+                  pm_strcat(full_path, "/");
+               }
             }
             ndmp_store_attribute_record(nis->jcr, full_path.c_str(), nis->virtual_filename, attribs.c_str(), FileType,
                                         0, (ndmp_fstat.fh_info.valid == NDMP9_VALIDITY_VALID) ? ndmp_fstat.fh_info.value : 0);
