@@ -32,7 +32,7 @@
 #include "dird.h"
 
 /* Forward referenced functions */
-static int update_volume(UAContext *ua);
+static bool update_volume(UAContext *ua);
 static bool update_pool(UAContext *ua);
 static bool update_job(UAContext *ua);
 static bool update_stats(UAContext *ua);
@@ -51,7 +51,7 @@ static void update_slots(UAContext *ua);
  *    update stats [days=...]
  *         updates long term statistics
  */
-int update_cmd(UAContext *ua, const char *cmd)
+bool update_cmd(UAContext *ua, const char *cmd)
 {
    static const char *kw[] = {
       NT_("media"),  /* 0 */
@@ -65,27 +65,27 @@ int update_cmd(UAContext *ua, const char *cmd)
    };
 
    if (!open_client_db(ua)) {
-      return 1;
+      return true;
    }
 
    switch (find_arg_keyword(ua, kw)) {
    case 0:
    case 1:
       update_volume(ua);
-      return 1;
+      return true;
    case 2:
       update_pool(ua);
-      return 1;
+      return true;
    case 3:
    case 4:
       update_slots(ua);
-      return 1;
+      return true;
    case 5:
       update_job(ua);
-      return 1;
+      return true;
    case 6:
       update_stats(ua);
-      return 1;
+      return true;
    default:
       break;
    }
@@ -111,7 +111,7 @@ int update_cmd(UAContext *ua, const char *cmd)
    default:
       break;
    }
-   return 1;
+   return true;
 }
 
 static void update_volstatus(UAContext *ua, const char *val, MEDIA_DBR *mr)
@@ -240,45 +240,48 @@ static void update_volmaxbytes(UAContext *ua, char *val, MEDIA_DBR *mr)
 
 static void update_volrecycle(UAContext *ua, char *val, MEDIA_DBR *mr)
 {
-   int recycle;
+   bool recycle;
    char ed1[50];
-
    POOL_MEM query(PM_MESSAGE);
+
    if (!is_yesno(val, &recycle)) {
       ua->error_msg(_("Invalid value. It must be yes or no.\n"));
       return;
    }
+
    Mmsg(query, "UPDATE Media SET Recycle=%d WHERE MediaId=%s",
-      recycle, edit_int64(mr->MediaId, ed1));
+        recycle ? 1 : 0, edit_int64(mr->MediaId, ed1));
+
    if (!db_sql_query(ua->db, query.c_str())) {
       ua->error_msg("%s", db_strerror(ua->db));
    } else {
       ua->info_msg(_("New Recycle flag is: %s\n"),
-         recycle==1?_("yes"):_("no"));
+                   recycle ? _("yes") : _("no"));
    }
 }
 
 static void update_volinchanger(UAContext *ua, char *val, MEDIA_DBR *mr)
 {
-   int InChanger;
    char ed1[50];
-
+   bool InChanger;
    POOL_MEM query(PM_MESSAGE);
+
    if (!is_yesno(val, &InChanger)) {
       ua->error_msg(_("Invalid value. It must be yes or no.\n"));
       return;
    }
+
    Mmsg(query, "UPDATE Media SET InChanger=%d WHERE MediaId=%s",
-      InChanger, edit_int64(mr->MediaId, ed1));
+        InChanger ? 1 : 0, edit_int64(mr->MediaId, ed1));
+
    if (!db_sql_query(ua->db, query.c_str())) {
       ua->error_msg("%s", db_strerror(ua->db));
    } else {
       ua->info_msg(_("New InChanger flag is: %s\n"),
-         InChanger==1?_("yes"):_("no"));
+                   InChanger ? _("yes") : _("no"));
    }
 }
 
-
 static void update_volslot(UAContext *ua, char *val, MEDIA_DBR *mr)
 {
    POOL_DBR pr;
@@ -297,7 +300,7 @@ static void update_volslot(UAContext *ua, char *val, MEDIA_DBR *mr)
    }
    /*
     * Make sure to use db_update... rather than doing this directly,
-    *   so that any Slot is handled correctly.
+    * so that any Slot is handled correctly.
     */
    set_storageid_in_mr(NULL, mr);
    if (!db_update_media_record(ua->jcr, ua->db, mr)) {
@@ -307,7 +310,9 @@ static void update_volslot(UAContext *ua, char *val, MEDIA_DBR *mr)
    }
 }
 
-/* Modify the Pool in which this Volume is located */
+/*
+ * Modify the Pool in which this Volume is located
+ */
 void update_vol_pool(UAContext *ua, char *val, MEDIA_DBR *mr, POOL_DBR *opr)
 {
    POOL_DBR pr;
@@ -320,8 +325,7 @@ void update_vol_pool(UAContext *ua, char *val, MEDIA_DBR *mr, POOL_DBR *opr)
       return;
    }
    mr->PoolId = pr.PoolId;            /* set new PoolId */
-   /*
-    */
+
    db_lock(ua->db);
    Mmsg(query, "UPDATE Media SET PoolId=%s WHERE MediaId=%s",
       edit_int64(mr->PoolId, ed1), edit_int64(mr->MediaId, ed2));
@@ -341,7 +345,9 @@ void update_vol_pool(UAContext *ua, char *val, MEDIA_DBR *mr, POOL_DBR *opr)
    db_unlock(ua->db);
 }
 
-/* Modify the RecyclePool of a Volume */
+/*
+ * Modify the RecyclePool of a Volume
+ */
 void update_vol_recyclepool(UAContext *ua, char *val, MEDIA_DBR *mr)
 {
    POOL_DBR pr;
@@ -350,17 +356,20 @@ void update_vol_recyclepool(UAContext *ua, char *val, MEDIA_DBR *mr)
    const char *poolname;
 
    if (val && *val) { /* update volume recyclepool="Scratch" */
-      /* If a pool name is given, look up the PoolId */
+      /*
+       * If a pool name is given, look up the PoolId
+       */
       memset(&pr, 0, sizeof(pr));
       bstrncpy(pr.Name, val, sizeof(pr.Name));
       if (!get_pool_dbr(ua, &pr, NT_("recyclepool"))) {
          return;
       }
-      /* pool = select_pool_resource(ua);  */
       mr->RecyclePoolId = pr.PoolId;            /* get the PoolId */
       poolname = pr.Name;
    } else { /* update volume recyclepool="" */
-      /* If no pool name is given, set the PoolId to 0 (the default) */
+      /*
+       * If no pool name is given, set the PoolId to 0 (the default)
+       */
       mr->RecyclePoolId = 0;
       poolname = _("*None*");
    }
@@ -376,6 +385,32 @@ void update_vol_recyclepool(UAContext *ua, char *val, MEDIA_DBR *mr)
    db_unlock(ua->db);
 }
 
+/*
+ * Modify the Storage in which this Volume is located
+ */
+void update_vol_storage(UAContext *ua, char *val, MEDIA_DBR *mr)
+{
+   STORAGE_DBR sr;
+   POOL_MEM query(PM_MESSAGE);
+   char ed1[50], ed2[50];
+
+   memset(&sr, 0, sizeof(sr));
+   bstrncpy(sr.Name, val, sizeof(sr.Name));
+   if (!get_storage_dbr(ua, &sr)) {
+      return;
+   }
+   mr->StorageId = sr.StorageId;            /* set new StorageId */
+
+   db_lock(ua->db);
+   Mmsg(query, "UPDATE Media SET StorageId=%s WHERE MediaId=%s",
+      edit_int64(mr->StorageId, ed1), edit_int64(mr->MediaId, ed2));
+   if (!db_sql_query(ua->db, query.c_str())) {
+      ua->error_msg("%s", db_strerror(ua->db));
+   }
+
+   db_unlock(ua->db);
+}
+
 /*
  * Refresh the Volume information from the Pool record
  */
@@ -386,7 +421,7 @@ static void update_vol_from_pool(UAContext *ua, MEDIA_DBR *mr)
    memset(&pr, 0, sizeof(pr));
    pr.PoolId = mr->PoolId;
    if (!db_get_pool_record(ua->jcr, ua->db, &pr) ||
-       !acl_access_ok(ua, Pool_ACL, pr.Name, true)) {
+       !ua->acl_access_ok(Pool_ACL, pr.Name, true)) {
       return;
    }
    set_pool_dbr_defaults_in_media_dbr(mr, &pr);
@@ -448,7 +483,7 @@ static void update_all_vols(UAContext *ua)
       /*
        * Check access to pool.
        */
-      if (!acl_access_ok(ua, Pool_ACL, pr.Name, false)) {
+      if (!ua->acl_access_ok(Pool_ACL, pr.Name, false)) {
          continue;
       }
 
@@ -504,7 +539,7 @@ static void update_vol_actiononpurge(UAContext *ua, char *val, MEDIA_DBR *mr)
  *  writing on the volume, set it to anything other
  *  than Append.
  */
-static int update_volume(UAContext *ua)
+static bool update_volume(UAContext *ua)
 {
    POOLRES *pool;
    POOLMEM *query;
@@ -529,6 +564,7 @@ static int update_volume(UAContext *ua)
       NT_("Enabled"),       /* 12 */
       NT_("RecyclePool"),   /* 13 */
       NT_("ActionOnPurge"), /* 14 */
+      NT_("Storage"),       /* 15 */
       NULL
    };
 
@@ -545,7 +581,7 @@ static int update_volume(UAContext *ua)
       if ((j = find_arg_with_value(ua, kw[i])) > 0) {
          /* If all from pool don't select a media record */
          if (i != AllFromPool && !select_media_dbr(ua, &mr)) {
-            return 0;
+            return false;
          }
          switch (i) {
          case 0:
@@ -585,10 +621,10 @@ static int update_volume(UAContext *ua)
             break;
          case 10:
             update_vol_from_pool(ua, &mr);
-            return 1;
+            return true;
          case 11:
             update_all_vols_from_pool(ua, ua->argv[j]);
-            return 1;
+            return true;
          case 12:
             update_volenabled(ua, ua->argv[j], &mr);
             break;
@@ -598,6 +634,9 @@ static int update_volume(UAContext *ua)
          case 14:
             update_vol_actiononpurge(ua, ua->argv[j], &mr);
             break;
+         case 15:
+            update_vol_storage(ua, ua->argv[j], &mr);
+            break;
          }
          done = true;
       }
@@ -606,15 +645,17 @@ static int update_volume(UAContext *ua)
    /* Allow user to simply update all volumes */
    if (find_arg(ua, NT_("fromallpools")) > 0) {
       update_all_vols(ua);
-      return 1;
+      return true;
    }
 
    for ( ; !done; ) {
       POOL_DBR pr;
       MEDIA_DBR mr;
+      STORAGE_DBR sr;
 
       memset(&pr, 0, sizeof(pr));
       memset(&mr, 0, sizeof(mr));
+      memset(&sr, 0, sizeof(sr));
 
       start_prompt(ua, _("Parameters to modify:\n"));
       add_prompt(ua, _("Volume Status"));              /* 0 */
@@ -634,14 +675,15 @@ static int update_volume(UAContext *ua)
       add_prompt(ua, _("Enabled")),                    /* 14 */
       add_prompt(ua, _("RecyclePool")),                /* 15 */
       add_prompt(ua, _("Action On Purge")),            /* 16 */
-      add_prompt(ua, _("Done"));                       /* 17 */
+      add_prompt(ua, _("Storage")),                    /* 17 */
+      add_prompt(ua, _("Done"));                       /* 18 */
       i = do_prompt(ua, "", _("Select parameter to modify"), NULL, 0);
 
       /* For All Volumes, All Volumes from Pool, and Done, we don't need
            * a Volume record */
-      if ( i != 12 && i != 13 && i != 17) {
+      if ( i != 12 && i != 13 && i != 18) {
          if (!select_media_dbr(ua, &mr)) {  /* Get Volume record */
-            return 0;
+            return false;
          }
          ua->info_msg(_("Updating Volume \"%s\"\n"), mr.VolumeName);
       }
@@ -661,7 +703,7 @@ static int update_volume(UAContext *ua)
          }
          add_prompt(ua, NT_("Read-Only"));
          if (do_prompt(ua, "", _("Choose new Volume Status"), ua->cmd, sizeof(mr.VolStatus)) < 0) {
-            return 1;
+            return true;
          }
          update_volstatus(ua, ua->cmd, &mr);
          break;
@@ -669,7 +711,7 @@ static int update_volume(UAContext *ua)
          ua->info_msg(_("Current retention period is: %s\n"),
             edit_utime(mr.VolRetention, ed1, sizeof(ed1)));
          if (!get_cmd(ua, _("Enter Volume Retention period: "))) {
-            return 0;
+            return false;
          }
          update_volretention(ua, ua->cmd, &mr);
          break;
@@ -678,7 +720,7 @@ static int update_volume(UAContext *ua)
          ua->info_msg(_("Current use duration is: %s\n"),
             edit_utime(mr.VolUseDuration, ed1, sizeof(ed1)));
          if (!get_cmd(ua, _("Enter Volume Use Duration: "))) {
-            return 0;
+            return false;
          }
          update_voluseduration(ua, ua->cmd, &mr);
          break;
@@ -686,7 +728,7 @@ static int update_volume(UAContext *ua)
       case 3:                         /* Max Jobs */
          ua->info_msg(_("Current max jobs is: %u\n"), mr.MaxVolJobs);
          if (!get_pint(ua, _("Enter new Maximum Jobs: "))) {
-            return 0;
+            return false;
          }
          update_volmaxjobs(ua, ua->cmd, &mr);
          break;
@@ -694,7 +736,7 @@ static int update_volume(UAContext *ua)
       case 4:                         /* Max Files */
          ua->info_msg(_("Current max files is: %u\n"), mr.MaxVolFiles);
          if (!get_pint(ua, _("Enter new Maximum Files: "))) {
-            return 0;
+            return false;
          }
          update_volmaxfiles(ua, ua->cmd, &mr);
          break;
@@ -702,7 +744,7 @@ static int update_volume(UAContext *ua)
       case 5:                         /* Max Bytes */
          ua->info_msg(_("Current value is: %s\n"), edit_uint64(mr.MaxVolBytes, ed1));
          if (!get_cmd(ua, _("Enter new Maximum Bytes: "))) {
-            return 0;
+            return false;
          }
          update_volmaxbytes(ua, ua->cmd, &mr);
          break;
@@ -710,9 +752,9 @@ static int update_volume(UAContext *ua)
 
       case 6:                         /* Recycle */
          ua->info_msg(_("Current recycle flag is: %s\n"),
-            mr.Recycle==1?_("yes"):_("no"));
+                      (mr.Recycle == 1) ? _("yes") : _("no"));
          if (!get_yesno(ua, _("Enter new Recycle status: "))) {
-            return 0;
+            return false;
          }
          update_volrecycle(ua, ua->cmd, &mr);
          break;
@@ -720,17 +762,16 @@ static int update_volume(UAContext *ua)
       case 7:                         /* Slot */
          ua->info_msg(_("Current Slot is: %d\n"), mr.Slot);
          if (!get_pint(ua, _("Enter new Slot: "))) {
-            return 0;
+            return false;
          }
          update_volslot(ua, ua->cmd, &mr);
          break;
 
       case 8:                         /* InChanger */
          ua->info_msg(_("Current InChanger flag is: %d\n"), mr.InChanger);
-         bsnprintf(buf, sizeof(buf), _("Set InChanger flag for Volume \"%s\": yes/no: "),
-            mr.VolumeName);
+         bsnprintf(buf, sizeof(buf), _("Set InChanger flag for Volume \"%s\": yes/no: "), mr.VolumeName);
          if (!get_yesno(ua, buf)) {
-            return 0;
+            return false;
          }
          mr.InChanger = ua->pint32_val;
          /*
@@ -752,12 +793,12 @@ static int update_volume(UAContext *ua)
                         "in loss of data on your Volume\n\n"));
          ua->info_msg(_("Current Volume Files is: %u\n"), mr.VolFiles);
          if (!get_pint(ua, _("Enter new number of Files for Volume: "))) {
-            return 0;
+            return false;
          }
          VolFiles = ua->pint32_val;
          if (VolFiles != (int)(mr.VolFiles + 1)) {
             ua->warning_msg(_("Normally, you should only increase Volume Files by one!\n"));
-            if (!get_yesno(ua, _("Increase Volume Files? (yes/no): ")) || ua->pint32_val == 0) {
+            if (!get_yesno(ua, _("Increase Volume Files? (yes/no): ")) || !ua->pint32_val) {
                break;
             }
          }
@@ -776,33 +817,33 @@ static int update_volume(UAContext *ua)
          pr.PoolId = mr.PoolId;
          if (!db_get_pool_record(ua->jcr, ua->db, &pr)) {
             ua->error_msg("%s", db_strerror(ua->db));
-            return 0;
+            return false;
          }
          ua->info_msg(_("Current Pool is: %s\n"), pr.Name);
          if (!get_cmd(ua, _("Enter new Pool name: "))) {
-            return 0;
+            return false;
          }
          update_vol_pool(ua, ua->cmd, &mr, &pr);
-         return 1;
+         return true;
 
       case 11:
          update_vol_from_pool(ua, &mr);
-         return 1;
+         return true;
       case 12:
          pool = select_pool_resource(ua);
          if (pool) {
             update_all_vols_from_pool(ua, pool->name());
          }
-         return 1;
+         return true;
 
       case 13:
          update_all_vols(ua);
-         return 1;
+         return true;
 
       case 14:
          ua->info_msg(_("Current Enabled is: %d\n"), mr.Enabled);
          if (!get_cmd(ua, _("Enter new Enabled: "))) {
-            return 0;
+            return false;
          }
 
          update_volenabled(ua, ua->cmd, &mr);
@@ -816,28 +857,42 @@ static int update_volume(UAContext *ua)
             ua->info_msg(_("No current RecyclePool\n"));
          }
          if (!select_pool_dbr(ua, &pr, NT_("recyclepool"))) {
-            return 0;
+            return false;
          }
          update_vol_recyclepool(ua, pr.Name, &mr);
-         return 1;
+         return true;
 
       case 16:
          pm_strcpy(ret, "");
          ua->info_msg(_("Current ActionOnPurge is: %s\n"),
                       action_on_purge_to_string(mr.ActionOnPurge, ret));
          if (!get_cmd(ua, _("Enter new ActionOnPurge (one of: Truncate, None): "))) {
-            return 0;
+            return false;
          }
 
          update_vol_actiononpurge(ua, ua->cmd, &mr);
          break;
 
+      case 17:
+         sr.StorageId = mr.StorageId;
+         if (db_get_storage_record(ua->jcr, ua->db, &sr)) {
+            ua->info_msg(_("Current Storage is: %s\n"), sr.Name);
+         } else {
+            ua->info_msg(_("Warning, could not find current Storage\n"));
+         }
+         if (!select_storage_dbr(ua, &sr, NT_("storage"))) {
+            return false;
+         }
+         update_vol_storage(ua, sr.Name, &mr);
+         ua->info_msg(_("New Storage is: %s\n"), sr.Name);
+         return true;
+
       default:                        /* Done or error */
          ua->info_msg(_("Selection terminated.\n"));
-         return 1;
+         return true;
       }
    }
-   return 1;
+   return true;
 }
 
 /*
@@ -890,31 +945,36 @@ static bool update_pool(UAContext *ua)
    }
    query = get_pool_memory(PM_MESSAGE);
    Mmsg(query, list_pool, edit_int64(pr.PoolId, ed1));
-   db_list_sql_query(ua->jcr, ua->db, query, printit, ua, true, HORZ_LIST);
+   db_list_sql_query(ua->jcr, ua->db, query, ua->send, HORZ_LIST, true);
    free_pool_memory(query);
    ua->info_msg(_("Pool DB record updated from resource.\n"));
    return true;
 }
 
 /*
- * Update a Job record -- allows you to change the
- *  date fields in a Job record. This helps when
- *  providing migration from other vendors.
+ * Update a Job record -- allows to change the fields in a Job record.
  */
 static bool update_job(UAContext *ua)
 {
    int i;
-   char ed1[50], ed2[50];
+   char ed1[50], ed2[50], ed3[50], ed4[50];
    POOL_MEM cmd(PM_MESSAGE);
    JOB_DBR jr;
    CLIENT_DBR cr;
    utime_t StartTime;
    char *client_name = NULL;
+   char *job_name = NULL;
    char *start_time = NULL;
+   char job_type = '\0';
+   DBId_t fileset_id = 0;
    const char *kw[] = {
       NT_("starttime"),                   /* 0 */
       NT_("client"),                      /* 1 */
-      NULL };
+      NT_("filesetid"),                   /* 2 */
+      NT_("jobname"),                     /* 3 */
+      NT_("jobtype"),                     /* 4 */
+      NULL
+   };
 
    Dmsg1(200, "cmd=%s\n", ua->cmd);
    i = find_arg_with_value(ua, NT_("jobid"));
@@ -934,18 +994,27 @@ static bool update_job(UAContext *ua)
       int j;
       if ((j=find_arg_with_value(ua, kw[i])) >= 0) {
          switch (i) {
-         case 0:                         /* start time */
+         case 0:                         /* Start time */
             start_time = ua->argv[j];
             break;
          case 1:                         /* Client name */
             client_name = ua->argv[j];
             break;
+         case 2:                         /* Fileset id */
+            fileset_id = str_to_uint64(ua->argv[j]);
+            break;
+         case 3:                         /* Job name */
+            job_name = ua->argv[j];
+            break;
+         case 4:                         /* Job Type */
+            job_type = ua->argv[j][0];
+            break;
          }
       }
    }
-   if (!client_name && !start_time) {
-      ua->error_msg(_("Neither Client nor StartTime specified.\n"));
-      return 0;
+   if (!client_name && !start_time && !fileset_id && !job_name && !job_type) {
+      ua->error_msg(_("Neither Client, StartTime, Filesetid, JobType nor Name specified.\n"));
+      return false;
    }
    if (client_name) {
       if (!get_client_dbr(ua, &cr)) {
@@ -953,6 +1022,15 @@ static bool update_job(UAContext *ua)
       }
       jr.ClientId = cr.ClientId;
    }
+   if (fileset_id) {
+      jr.FileSetId = fileset_id;
+   }
+   if (job_name) {
+      bstrncpy(jr.Name, job_name, MAX_NAME_LENGTH);
+   }
+   if (job_type) {
+      jr.JobType = job_type;
+   }
    if (start_time) {
       utime_t delta_start;
 
@@ -973,14 +1051,17 @@ static bool update_job(UAContext *ua)
       bstrutime(jr.cSchedTime, sizeof(jr.cSchedTime), jr.SchedTime);
       bstrutime(jr.cEndTime, sizeof(jr.cEndTime), jr.EndTime);
    }
-   Mmsg(cmd, "UPDATE Job SET ClientId=%s,StartTime='%s',SchedTime='%s',"
-             "EndTime='%s',JobTDate=%s WHERE JobId=%s",
+   Mmsg(cmd, "UPDATE Job SET Name='%s', ClientId=%s,StartTime='%s',SchedTime='%s',"
+             "EndTime='%s',JobTDate=%s, FileSetId='%s', Type='%c' WHERE JobId=%s",
+             jr.Name,
              edit_int64(jr.ClientId, ed1),
              jr.cStartTime,
              jr.cSchedTime,
              jr.cEndTime,
-             edit_uint64(jr.JobTDate, ed1),
-             edit_int64(jr.JobId, ed2));
+             edit_uint64(jr.JobTDate, ed2),
+             edit_uint64(jr.FileSetId, ed3),
+             jr.JobType,
+             edit_int64(jr.JobId, ed4));
    if (!db_sql_query(ua->db, cmd.c_str())) {
       ua->error_msg("%s", db_strerror(ua->db));
       return false;
@@ -995,12 +1076,12 @@ static void update_slots(UAContext *ua)
 {
    USTORERES store;
    vol_list_t *vl;
-   dlist *vol_list = NULL;
+   changer_vol_list_t *vol_list = NULL;
    MEDIA_DBR mr;
    char *slot_list;
    bool scan;
-   int max_slots;
-   int drive = -1;
+   slot_number_t max_slots;
+   drive_number_t drive = -1;
    int Enabled = VOL_ENABLED;
    bool have_enabled;
    int i;
@@ -1029,7 +1110,7 @@ static void update_slots(UAContext *ua)
       have_enabled = false;
    }
 
-   max_slots = get_num_slots_from_SD(ua);
+   max_slots = get_num_slots(ua, ua->jcr->res.wstore);
    Dmsg1(100, "max_slots=%d\n", max_slots);
    if (max_slots <= 0) {
       ua->warning_msg(_("No slots in changer to scan.\n"));
@@ -1043,7 +1124,7 @@ static void update_slots(UAContext *ua)
       return;
    }
 
-   vol_list = get_vol_list_from_SD(ua, store.store, false, scan);
+   vol_list = get_vol_list_from_storage(ua, store.store, false, scan, false);
    if (!vol_list) {
       ua->warning_msg(_("No Volumes found to update, or no barcodes.\n"));
       goto bail_out;
@@ -1058,7 +1139,7 @@ static void update_slots(UAContext *ua)
     * Walk through the list updating the media records
     */
    memset(&mr, 0, sizeof(mr));
-   foreach_dlist(vl, vol_list) {
+   foreach_dlist(vl, vol_list->contents) {
       if (vl->Slot > max_slots) {
          ua->warning_msg(_("Slot %d greater than max %d ignored.\n"), vl->Slot, max_slots);
          continue;
@@ -1163,7 +1244,7 @@ static void update_slots(UAContext *ua)
 
 bail_out:
    if (vol_list) {
-      free_vol_list(vol_list);
+      storage_release_vol_list(store.store, vol_list);
    }
    free(slot_list);
    close_sd_bsock(ua);
@@ -1182,7 +1263,7 @@ bail_out:
  *
  * The vol_list passed here needs to be from an "autochanger listall" cmd.
  */
-void update_slots_from_vol_list(UAContext *ua, STORERES *store, dlist *vol_list, char *slot_list)
+void update_slots_from_vol_list(UAContext *ua, STORERES *store, changer_vol_list_t *vol_list, char *slot_list)
 {
    vol_list_t *vl;
    MEDIA_DBR mr;
@@ -1194,7 +1275,7 @@ void update_slots_from_vol_list(UAContext *ua, STORERES *store, dlist *vol_list,
    /*
     * Walk through the list updating the media records
     */
-   foreach_dlist(vl, vol_list) {
+   foreach_dlist(vl, vol_list->contents) {
       /*
        * We are only interested in normal slots.
        */
@@ -1287,7 +1368,7 @@ void update_slots_from_vol_list(UAContext *ua, STORERES *store, dlist *vol_list,
  *
  * The vol_list passed here needs to be from an "autochanger listall" cmd.
  */
-void update_inchanger_for_export(UAContext *ua, STORERES *store, dlist *vol_list, char *slot_list)
+void update_inchanger_for_export(UAContext *ua, STORERES *store, changer_vol_list_t *vol_list, char *slot_list)
 {
    vol_list_t *vl;
    MEDIA_DBR mr;
@@ -1299,7 +1380,7 @@ void update_inchanger_for_export(UAContext *ua, STORERES *store, dlist *vol_list
    /*
     * Walk through the list updating the media records
     */
-   foreach_dlist(vl, vol_list) {
+   foreach_dlist(vl, vol_list->contents) {
       /*
        * We are only interested in normal slots.
        */
