@@ -69,13 +69,11 @@ static void close_vss_backup_session(JCR *jcr);
  * also run a "heartbeat" monitor which reads the socket and
  * reacts accordingly (at the moment it has nothing to do
  * except echo the heartbeat to the Director).
- *
  */
-bool blast_data_to_storage_daemon(JCR *jcr, char *addr)
+bool blast_data_to_storage_daemon(JCR *jcr, char *addr, crypto_cipher_t cipher)
 {
    BSOCK *sd;
    bool ok = true;
-   // TODO landonf: Allow user to specify encryption algorithm
 
    sd = jcr->store_bsock;
 
@@ -104,13 +102,15 @@ bool blast_data_to_storage_daemon(JCR *jcr, char *addr)
       return false;
    }
 
-   if (!crypto_session_start(jcr)) {
+   if (!crypto_session_start(jcr, cipher)) {
       return false;
    }
 
    set_find_options((FF_PKT *)jcr->ff, jcr->incremental, jcr->mtime);
 
-   /** in accurate mode, we overload the find_one check function */
+   /**
+    * In accurate mode, we overload the find_one check function
+    */
    if (jcr->accurate) {
       set_find_changed_function((FF_PKT *)jcr->ff, accurate_check_file);
    }
@@ -133,7 +133,9 @@ bool blast_data_to_storage_daemon(JCR *jcr, char *addr)
       jcr->xattr_data->u.build->content = get_pool_memory(PM_MESSAGE);
    }
 
-   /** Subroutine save_file() is called for each file */
+   /**
+    * Subroutine save_file() is called for each file
+    */
    if (!find_files(jcr, (FF_PKT *)jcr->ff, save_file, plugin_save)) {
       ok = false;                     /* error */
       jcr->setJobStatus(JS_ErrorTerminated);
@@ -492,13 +494,13 @@ static inline bool do_backup_xattr(JCR *jcr, FF_PKT *ff_pkt)
 
 /**
  * Called here by find() for each file included.
- *   This is a callback. The original is find_files() above.
+ * This is a callback. The original is find_files() above.
  *
- *  Send the file and its data to the Storage daemon.
+ * Send the file and its data to the Storage daemon.
  *
- *  Returns: 1 if OK
- *           0 if error
- *          -1 to ignore file/directory (not used here)
+ * Returns: 1 if OK
+ *          0 if error
+ *         -1 to ignore file/directory (not used here)
  */
 int save_file(JCR *jcr, FF_PKT *ff_pkt, bool top_level)
 {
@@ -1572,33 +1574,38 @@ static void close_vss_backup_session(JCR *jcr)
     * STOP VSS ON WIN32
     * Tell vss to close the backup session
     */
-   if (jcr->VSS) {
-      if (g_pVSSClient->CloseBackup()) {
+   if (jcr->pVSSClient) {
+      /*
+       * We are about to call the BackupComplete VSS method so let all plugins know
+       * that by raising the bEventVssBackupComplete event.
+       */
+      generate_plugin_event(jcr, bEventVssBackupComplete);
+      if (jcr->pVSSClient->CloseBackup()) {
          /*
           * Inform user about writer states
           */
-         for (int i=0; i<(int)g_pVSSClient->GetWriterCount(); i++) {
+         for (int i = 0; i < (int)jcr->pVSSClient->GetWriterCount(); i++) {
             int msg_type = M_INFO;
-            if (g_pVSSClient->GetWriterState(i) < 1) {
+            if (jcr->pVSSClient->GetWriterState(i) < 1) {
                msg_type = M_WARNING;
                jcr->JobErrors++;
             }
-            Jmsg(jcr, msg_type, 0, _("VSS Writer (BackupComplete): %s\n"), g_pVSSClient->GetWriterInfo(i));
+            Jmsg(jcr, msg_type, 0, _("VSS Writer (BackupComplete): %s\n"), jcr->pVSSClient->GetWriterInfo(i));
          }
       }
 
       /*
        * Generate Job global writer metadata
        */
-      WCHAR *metadata = g_pVSSClient->GetMetadata();
+      wchar_t *metadata = jcr->pVSSClient->GetMetadata();
       if (metadata) {
          FF_PKT *ff_pkt = jcr->ff;
          ff_pkt->fname = (char *)"*all*"; /* for all plugins */
          ff_pkt->type = FT_RESTORE_FIRST;
          ff_pkt->LinkFI = 0;
          ff_pkt->object_name = (char *)"job_metadata.xml";
-         ff_pkt->object = (char *)metadata;
-         ff_pkt->object_len = (wcslen(metadata) + 1) * sizeof(WCHAR);
+         ff_pkt->object = BSTR_2_str(metadata);
+         ff_pkt->object_len = (wcslen(metadata) + 1) * sizeof(wchar_t);
          ff_pkt->object_index = (int)time(NULL);
          save_file(jcr, ff_pkt, true);
      }
