@@ -3,7 +3,7 @@
 
    Copyright (C) 2007-2012 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2014 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2016 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -40,9 +40,9 @@ const char *plugin_type = "-fd.dll";
 const char *plugin_type = "-fd.so";
 #endif
 static alist *fd_plugin_list = NULL;
+static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 
 extern int save_file(JCR *jcr, FF_PKT *ff_pkt, bool top_level);
-extern bool check_changes(JCR *jcr, FF_PKT *ff_pkt);
 
 /*
  * Function pointers to be set here
@@ -59,14 +59,15 @@ extern DLL_IMP_EXP boffset_t (*plugin_blseek)(BFILE *bfd, boffset_t offset, int
 static bRC bareosGetValue(bpContext *ctx, bVariable var, void *value);
 static bRC bareosSetValue(bpContext *ctx, bVariable var, void *value);
 static bRC bareosRegisterEvents(bpContext *ctx, int nr_events, ...);
-static bRC bareosJobMsg(bpContext *ctx, const char *file, int line,
+static bRC bareosUnRegisterEvents(bpContext *ctx, int nr_events, ...);
+static bRC bareosJobMsg(bpContext *ctx, const char *fname, int line,
                         int type, utime_t mtime, const char *fmt, ...);
-static bRC bareosDebugMsg(bpContext *ctx, const char *file, int line,
+static bRC bareosDebugMsg(bpContext *ctx, const char *fname, int line,
                           int level, const char *fmt, ...);
-static void *bareosMalloc(bpContext *ctx, const char *file, int line,
+static void *bareosMalloc(bpContext *ctx, const char *fname, int line,
                           size_t size);
 static void bareosFree(bpContext *ctx, const char *file, int line, void *mem);
-static bRC  bareosAddExclude(bpContext *ctx, const char *file);
+static bRC bareosAddExclude(bpContext *ctx, const char *file);
 static bRC bareosAddInclude(bpContext *ctx, const char *file);
 static bRC bareosAddOptions(bpContext *ctx, const char *opts);
 static bRC bareosAddRegex(bpContext *ctx, const char *item, int type);
@@ -78,6 +79,9 @@ static bool is_plugin_compatible(Plugin *plugin);
 static bool get_plugin_name(JCR *jcr, char *cmd, int *ret);
 static bRC bareosCheckChanges(bpContext *ctx, struct save_pkt *sp);
 static bRC bareosAcceptFile(bpContext *ctx, struct save_pkt *sp);
+static bRC bareosSetSeenBitmap(bpContext *ctx, bool all, char *fname);
+static bRC bareosClearSeenBitmap(bpContext *ctx, bool all, char *fname);
+static bRC bareosGetInstanceCount(bpContext *ctx, int *ret);
 
 /*
  * These will be plugged into the global pointer structure for the findlib.
@@ -99,6 +103,8 @@ static bFuncs bfuncs = {
    sizeof(bFuncs),
    FD_PLUGIN_INTERFACE_VERSION,
    bareosRegisterEvents,
+   bareosUnRegisterEvents,
+   bareosGetInstanceCount,
    bareosGetValue,
    bareosSetValue,
    bareosJobMsg,
@@ -114,7 +120,9 @@ static bFuncs bfuncs = {
    bareosNewInclude,
    bareosNewPreInclude,
    bareosCheckChanges,
-   bareosAcceptFile
+   bareosAcceptFile,
+   bareosSetSeenBitmap,
+   bareosClearSeenBitmap
 };
 
 /*
@@ -129,6 +137,7 @@ struct b_plugin_ctx {
    char events[nbytes_for_bits(FD_NR_EVENTS + 1)]; /* enabled events bitmask */
    findINCEXE *exclude;                            /* pointer to exclude files */
    findINCEXE *include;                            /* pointer to include/exclude files */
+   Plugin *plugin;                                 /* pointer to plugin of which this is an instance off */
 };
 
 static inline bool is_event_enabled(bpContext *ctx, bEventType eventType)
@@ -163,6 +172,25 @@ static inline bool is_plugin_disabled(bpContext *ctx)
    return b_ctx->disabled;
 }
 
+static bool is_ctx_good(bpContext *ctx, JCR *&jcr, b_plugin_ctx *&bctx)
+{
+   if (!ctx) {
+      return false;
+   }
+
+   bctx = (b_plugin_ctx *)ctx->bContext;
+   if (!bctx) {
+      return false;
+   }
+
+   jcr = bctx->jcr;
+   if (!jcr) {
+      return false;
+   }
+
+   return true;
+}
+
 /**
  * Test if event is for this plugin
  */
@@ -197,15 +225,27 @@ static bool for_this_plugin(Plugin *plugin, char *name, int len)
 /**
  * Raise a certain plugin event.
  */
-static inline bRC trigger_plugin_event(JCR *jcr, bEventType eventType, bEvent *event, bpContext *ctx, void *value)
+static inline bool trigger_plugin_event(JCR *jcr, bEventType eventType,
+                                        bEvent *event, bpContext *ctx,
+                                        void *value, alist *plugin_ctx_list,
+                                        int *index, bRC *rc)
+
 {
+   bool stop = false;
+
    if (!is_event_enabled(ctx, eventType)) {
       Dmsg1(dbglvl, "Event %d disabled for this plugin.\n", eventType);
-      return bRC_OK;
+      if (rc) {
+         *rc = bRC_OK;
+      }
+      goto bail_out;
    }
 
    if (is_plugin_disabled(ctx)) {
-      return bRC_OK;
+      if (rc) {
+         *rc = bRC_OK;
+      }
+      goto bail_out;
    }
 
    if (eventType == bEventEndRestoreJob) {
@@ -222,13 +262,56 @@ static inline bRC trigger_plugin_event(JCR *jcr, bEventType eventType, bEvent *e
       }
    }
 
-   return plug_func(ctx->plugin)->handlePluginEvent(ctx, event, value);
+   /*
+    * See if we should care about the return code.
+    */
+   if (rc) {
+      *rc = plug_func(ctx->plugin)->handlePluginEvent(ctx, event, value);
+      switch (*rc) {
+      case bRC_OK:
+         break;
+      case bRC_Stop:
+      case bRC_Error:
+         stop = true;
+         break;
+      case bRC_More:
+         break;
+      case bRC_Term:
+         /*
+          * Request to unload this plugin.
+          * As we remove the plugin from the list of plugins we decrement
+          * the running index value so the next plugin gets triggered as
+          * that moved back a position in the alist.
+          */
+         if (index) {
+            unload_plugin(plugin_ctx_list, ctx->plugin, *index);
+            *index = ((*index) - 1);
+         }
+         break;
+      case bRC_Seen:
+         break;
+      case bRC_Core:
+         break;
+      case bRC_Skip:
+         stop = true;
+         break;
+      case bRC_Cancel:
+         break;
+      default:
+         break;
+      }
+   } else {
+      plug_func(ctx->plugin)->handlePluginEvent(ctx, event, value);
+   }
+
+bail_out:
+   return stop;
 }
 
 /**
- * Create a plugin event When receiving bEventCancelCommand, this function is called by an other thread.
+ * Create a plugin event When receiving bEventCancelCommand, this function is called by another thread.
  */
-void generate_plugin_event(JCR *jcr, bEventType eventType, void *value, bool reverse)
+bRC generate_plugin_event(JCR *jcr, bEventType eventType, void *value, bool reverse)
 {
    bEvent event;
    char *name = NULL;
@@ -238,9 +321,21 @@ void generate_plugin_event(JCR *jcr, bEventType eventType, void *value, bool rev
    restore_object_pkt *rop;
    bpContext *ctx;
    alist *plugin_ctx_list;
+   bRC rc = bRC_OK;
 
-   if (!fd_plugin_list || !jcr || !jcr->plugin_ctx_list) {
-      return;                         /* Return if no plugins loaded */
+   if (!fd_plugin_list) {
+      Dmsg0(dbglvl, "No bplugin_list: generate_plugin_event ignored.\n");
+      goto bail_out;
+   }
+
+   if (!jcr) {
+      Dmsg0(dbglvl, "No jcr: generate_plugin_event ignored.\n");
+      goto bail_out;
+   }
+
+   if (!jcr->plugin_ctx_list) {
+      Dmsg0(dbglvl, "No plugin_ctx_list: generate_plugin_event ignored.\n");
+      goto bail_out;
    }
 
    plugin_ctx_list = jcr->plugin_ctx_list;
@@ -253,7 +348,7 @@ void generate_plugin_event(JCR *jcr, bEventType eventType, void *value, bool rev
    case bEventOptionPlugin:
       name = (char *)value;
       if (!get_plugin_name(jcr, name, &len)) {
-         return;
+         goto bail_out;
       }
       break;
    case bEventRestoreObject:
@@ -268,7 +363,7 @@ void generate_plugin_event(JCR *jcr, bEventType eventType, void *value, bool rev
          if (*rop->plugin_name) {
             name = rop->plugin_name;
             if (!get_plugin_name(jcr, name, &len)) {
-               return;
+               goto bail_out;
             }
          }
       }
@@ -294,7 +389,7 @@ void generate_plugin_event(JCR *jcr, bEventType eventType, void *value, bool rev
     * If call_if_canceled is set, we call the plugin anyway
     */
    if (!call_if_canceled && jcr->is_job_canceled()) {
-      return;
+      goto bail_out;
    }
 
    event.eventType = eventType;
@@ -314,7 +409,9 @@ void generate_plugin_event(JCR *jcr, bEventType eventType, void *value, bool rev
             continue;
          }
 
-         trigger_plugin_event(jcr, eventType, &event, ctx, value);
+         if (trigger_plugin_event(jcr, eventType, &event, ctx, value, plugin_ctx_list, &i, &rc)) {
+            break;
+         }
       }
    } else {
       foreach_alist_index(i, ctx, plugin_ctx_list) {
@@ -323,11 +420,19 @@ void generate_plugin_event(JCR *jcr, bEventType eventType, void *value, bool rev
             continue;
          }
 
-         trigger_plugin_event(jcr, eventType, &event, ctx, value);
+         if (trigger_plugin_event(jcr, eventType, &event, ctx, value, plugin_ctx_list, &i, &rc)) {
+            break;
+         }
       }
    }
 
-   return;
+   if (jcr->is_job_canceled()) {
+      Dmsg0(dbglvl, "Cancel return from generate_plugin_event\n");
+      rc = bRC_Cancel;
+   }
+
+bail_out:
+   return rc;
 }
 
 /**
@@ -337,7 +442,7 @@ bool plugin_check_file(JCR *jcr, char *fname)
 {
    bpContext *ctx;
    alist *plugin_ctx_list;
-   int ret = bRC_OK;
+   int retval = bRC_OK;
 
    if (!fd_plugin_list || !jcr || !jcr->plugin_ctx_list || jcr->is_job_canceled()) {
       return false;                      /* Return if no plugins loaded */
@@ -359,13 +464,13 @@ bool plugin_check_file(JCR *jcr, char *fname)
          continue;
       }
 
-      ret = plug_func(ctx->plugin)->checkFile(ctx, fname);
-      if (ret == bRC_Seen) {
+      retval = plug_func(ctx->plugin)->checkFile(ctx, fname);
+      if (retval == bRC_Seen) {
          break;
       }
    }
 
-   return ret == bRC_Seen;
+   return retval == bRC_Seen;
 }
 
 /**
@@ -454,7 +559,7 @@ bRC plugin_option_handle_file(JCR *jcr, FF_PKT *ff_pkt, struct save_pkt *sp)
 {
    int len;
    char *cmd;
-   bRC ret = bRC_Core;
+   bRC retval = bRC_Core;
    bEvent event;
    bEventType eventType;
    bpContext *ctx;
@@ -505,13 +610,13 @@ bRC plugin_option_handle_file(JCR *jcr, FF_PKT *ff_pkt, struct save_pkt *sp)
       }
 
       jcr->plugin_ctx = ctx;
-      ret = plug_func(ctx->plugin)->handlePluginEvent(ctx, &event, sp);
+      retval = plug_func(ctx->plugin)->handlePluginEvent(ctx, &event, sp);
 
       goto bail_out;
    } /* end foreach loop */
 
 bail_out:
-   return ret;
+   return retval;
 }
 
 /**
@@ -532,7 +637,7 @@ bail_out:
 int plugin_save(JCR *jcr, FF_PKT *ff_pkt, bool top_level)
 {
    int len;
-   bRC ret;
+   bRC retval;
    char *cmd;
    bEvent event;
    bpContext *ctx;
@@ -726,12 +831,12 @@ int plugin_save(JCR *jcr, FF_PKT *ff_pkt, bool top_level)
           */
          copy_bits(FO_MAX, flags, ff_pkt->flags);
 
-         ret = plug_func(ctx->plugin)->endBackupFile(ctx);
-         if (ret == bRC_More || ret == bRC_OK) {
+         retval = plug_func(ctx->plugin)->endBackupFile(ctx);
+         if (retval == bRC_More || retval == bRC_OK) {
             accurate_mark_file_as_seen(jcr, fname.c_str());
          }
 
-         if (ret == bRC_More) {
+         if (retval == bRC_More) {
             continue;
          }
 
@@ -889,12 +994,12 @@ int plugin_estimate(JCR *jcr, FF_PKT *ff_pkt, bool top_level)
             Dmsg2(dbglvl, "index=%d object=%s\n", sp.index, sp.object);
          }
 
-         bRC ret = plug_func(ctx->plugin)->endBackupFile(ctx);
-         if (ret == bRC_More || ret == bRC_OK) {
+         bRC retval = plug_func(ctx->plugin)->endBackupFile(ctx);
+         if (retval == bRC_More || retval == bRC_OK) {
             accurate_mark_file_as_seen(jcr, sp.fname);
          }
 
-         if (ret == bRC_More) {
+         if (retval == bRC_More) {
             continue;
          }
 
@@ -1091,7 +1196,7 @@ bail_out:
 int plugin_create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
 {
    int flags;
-   int ret;
+   int retval;
    int status;
    Plugin *plugin;
    struct restore_pkt rp;
@@ -1134,30 +1239,28 @@ int plugin_create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
       return CF_ERROR;
    }
 
-   ret = plug_func(plugin)->createFile(ctx, &rp);
-   if (ret != bRC_OK) {
-      Qmsg2(jcr, M_ERROR, 0, _("Plugin createFile call failed. Stat=%d file=%s\n"), ret, attr->ofname);
+   retval = plug_func(plugin)->createFile(ctx, &rp);
+   if (retval != bRC_OK) {
+      Qmsg2(jcr, M_ERROR, 0, _("Plugin createFile call failed. Stat=%d file=%s\n"), retval, attr->ofname);
       return CF_ERROR;
    }
 
-   if (rp.create_status == CF_ERROR) {
+   switch (rp.create_status) {
+   case CF_ERROR:
       Qmsg1(jcr, M_ERROR, 0, _("Plugin createFile call failed. Returned CF_ERROR file=%s\n"), attr->ofname);
-      return CF_ERROR;
-   }
-
-   if (rp.create_status == CF_SKIP) {
-      return CF_SKIP;
-   }
-
-   if (rp.create_status == CF_CORE) {
-      return CF_CORE;           /* Let Bareos core handle the file creation */
-   }
-
-   /*
-    * Created link or directory?
-    */
-   if (rp.create_status == CF_CREATED) {
-      return rp.create_status;        /* yes, no need to bopen */
+      /*
+       * FALLTHROUGH
+       */
+   case CF_SKIP:
+      /*
+       * FALLTHROUGH
+       */
+   case CF_CORE:
+      /*
+       * FALLTHROUGH
+       */
+   case CF_CREATED:
+      return rp.create_status;
    }
 
    flags =  O_WRONLY | O_CREAT | O_TRUNC | O_BINARY;
@@ -1168,6 +1271,7 @@ int plugin_create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
 
    if (status < 0) {
       berrno be;
+
       be.set_errno(bfd->berrno);
       Qmsg2(jcr, M_ERROR, 0, _("Could not create %s: ERR=%s\n"),
             attr->ofname, be.bstrerror());
@@ -1224,7 +1328,7 @@ bool plugin_set_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
       if (is_bopen(ofd)) {
          bclose(ofd);
       }
-      pm_strcpy(attr->ofname, "*none*");
+      pm_strcpy(attr->ofname, "*None*");
    }
 
    return true;
@@ -1659,8 +1763,9 @@ static inline bpContext *instantiate_plugin(JCR *jcr, Plugin *plugin, char insta
    b_plugin_ctx *b_ctx;
 
    b_ctx = (b_plugin_ctx *)malloc(sizeof(b_plugin_ctx));
-   memset(b_ctx, 0, sizeof(b_plugin_ctx));
+   (b_plugin_ctx *)memset(b_ctx, 0, sizeof(b_plugin_ctx));
    b_ctx->jcr = jcr;
+   b_ctx->plugin = plugin;
 
    ctx = (bpContext *)malloc(sizeof(bpContext));
    ctx->instance = instance;
@@ -1723,7 +1828,7 @@ void free_plugins(JCR *jcr)
       return;
    }
 
-   Dmsg2(dbglvl, "Free instance dir-plugin_ctx_list=%p JobId=%d\n", jcr->plugin_ctx_list, jcr->JobId);
+   Dmsg2(dbglvl, "Free instance fd-plugin_ctx_list=%p JobId=%d\n", jcr->plugin_ctx_list, jcr->JobId);
    foreach_alist(ctx, jcr->plugin_ctx_list) {
       /*
        * Free the plugin instance
@@ -2000,24 +2105,15 @@ static bRC bareosGetValue(bpContext *ctx, bVariable var, void *value)
          break;
       case bVarFileSeen:
          break;                 /* a write only variable, ignore read request */
-      case bVarVssObject:
+      case bVarVssClient:
 #ifdef HAVE_WIN32
-         if (g_pVSSClient) {
-            *(void **)value = g_pVSSClient->GetVssObject();
-            Dmsg1(dbglvl, "fd-plugin: return bVarVssObject=%p\n", *(void **)value);
+         if (jcr->pVSSClient) {
+            *(void **)value = jcr->pVSSClient;
+            Dmsg1(dbglvl, "fd-plugin: return bVarVssClient=%p\n", *(void **)value);
             break;
-          }
+         }
 #endif
-          return bRC_Error;
-      case bVarVssDllHandle:
-#ifdef HAVE_WIN32
-         if (g_pVSSClient) {
-            *(void **)value = g_pVSSClient->GetVssDllHandle();
-            Dmsg1(dbglvl, "fd-plugin: return bVarVssDllHandle=%p\n", *(void **)value);
-            break;
-          }
-#endif
-       return bRC_Error;
+         return bRC_Error;
       case bVarWhere:
          *(char **)value = jcr->where;
          Dmsg1(dbglvl, "fd-plugin: return bVarWhere=%s\n", NPRT(*((char **)value)));
@@ -2052,6 +2148,9 @@ static bRC bareosSetValue(bpContext *ctx, bVariable var, void *value)
    }
 
    switch (var) {
+   case bVarLevel:
+      jcr->setJobLevel(*(int *)value);
+      break;
    case bVarFileSeen:
       if (!accurate_mark_file_as_seen(jcr, (char *)value)) {
          return bRC_Error;
@@ -2079,7 +2178,7 @@ static bRC bareosRegisterEvents(bpContext *ctx, int nr_events, ...)
    va_start(args, nr_events);
    for (i = 0; i < nr_events; i++) {
       event = va_arg(args, uint32_t);
-      Dmsg1(dbglvl, "Plugin wants event=%u\n", event);
+      Dmsg1(dbglvl, "fd-plugin: Plugin registered event=%u\n", event);
       set_bit(event, b_ctx->events);
    }
    va_end(args);
@@ -2087,7 +2186,68 @@ static bRC bareosRegisterEvents(bpContext *ctx, int nr_events, ...)
    return bRC_OK;
 }
 
-static bRC bareosJobMsg(bpContext *ctx, const char *file, int line,
+/**
+ * Get the number of instaces instantiated of a certain plugin.
+ */
+static bRC bareosGetInstanceCount(bpContext *ctx, int *ret)
+{
+   int cnt;
+   JCR *jcr, *njcr;
+   bpContext *nctx;
+   b_plugin_ctx *bctx;
+   bRC retval = bRC_Error;
+
+   if (!is_ctx_good(ctx, jcr, bctx)) {
+      goto bail_out;
+   }
+
+   P(mutex);
+
+   cnt = 0;
+   foreach_jcr(njcr) {
+      if (jcr->plugin_ctx_list) {
+         foreach_alist(nctx, jcr->plugin_ctx_list) {
+            if (nctx->plugin == bctx->plugin) {
+               cnt++;
+            }
+         }
+      }
+   }
+   endeach_jcr(njcr);
+
+   V(mutex);
+
+   *ret = cnt;
+   retval = bRC_OK;
+
+bail_out:
+   return retval;
+}
+
+static bRC bareosUnRegisterEvents(bpContext *ctx, int nr_events, ...)
+{
+   int i;
+   va_list args;
+   uint32_t event;
+   b_plugin_ctx *b_ctx;
+
+   if (!ctx) {
+      return bRC_Error;
+   }
+   b_ctx = (b_plugin_ctx *)ctx->bContext;
+
+   va_start(args, nr_events);
+   for (i = 0; i < nr_events; i++) {
+      event = va_arg(args, uint32_t);
+      Dmsg1(dbglvl, "fd-plugin: Plugin unregistered event=%u\n", event);
+      clear_bit(event, b_ctx->events);
+   }
+   va_end(args);
+
+   return bRC_OK;
+}
+
+static bRC bareosJobMsg(bpContext *ctx, const char *fname, int line,
                         int type, utime_t mtime, const char *fmt, ...)
 {
    JCR *jcr;
@@ -2108,7 +2268,7 @@ static bRC bareosJobMsg(bpContext *ctx, const char *file, int line,
    return bRC_OK;
 }
 
-static bRC bareosDebugMsg(bpContext *ctx, const char *file, int line,
+static bRC bareosDebugMsg(bpContext *ctx, const char *fname, int line,
                           int level, const char *fmt, ...)
 {
    va_list arg_ptr;
@@ -2117,61 +2277,42 @@ static bRC bareosDebugMsg(bpContext *ctx, const char *file, int line,
    va_start(arg_ptr, fmt);
    buffer.bvsprintf(fmt, arg_ptr);
    va_end(arg_ptr);
-   d_msg(file, line, level, "%s", buffer.c_str());
+   d_msg(fname, line, level, "%s", buffer.c_str());
 
    return bRC_OK;
 }
 
-static void *bareosMalloc(bpContext *ctx, const char *file, int line,
-                          size_t size)
+static void *bareosMalloc(bpContext *ctx, const char *fname, int line, size_t size)
 {
 #ifdef SMARTALLOC
-   return sm_malloc(file, line, size);
+   return sm_malloc(fname, line, size);
 #else
    return malloc(size);
 #endif
 }
 
-static void bareosFree(bpContext *ctx, const char *file, int line, void *mem)
+static void bareosFree(bpContext *ctx, const char *fname, int line, void *mem)
 {
 #ifdef SMARTALLOC
-   sm_free(file, line, mem);
+   sm_free(fname, line, mem);
 #else
    free(mem);
 #endif
 }
 
-static bool is_ctx_good(bpContext *ctx, JCR *&jcr, b_plugin_ctx *&bctx)
-{
-   if (!ctx) {
-      return false;
-   }
-
-   bctx = (b_plugin_ctx *)ctx->bContext;
-   if (!bctx) {
-      return false;
-   }
-
-   jcr = bctx->jcr;
-   if (!jcr) {
-      return false;
-   }
-
-   return true;
-}
-
 /**
  * Let the plugin define files/directories to be excluded from the main backup.
  */
-static bRC bareosAddExclude(bpContext *ctx, const char *file)
+static bRC bareosAddExclude(bpContext *ctx, const char *fname)
 {
    JCR *jcr;
    findINCEXE *old;
    b_plugin_ctx *bctx;
+
    if (!is_ctx_good(ctx, jcr, bctx)) {
       return bRC_Error;
    }
-   if (!file) {
+   if (!fname) {
       return bRC_Error;
    }
 
@@ -2196,14 +2337,14 @@ static bRC bareosAddExclude(bpContext *ctx, const char *file)
     */
    set_incexe(jcr, bctx->exclude);
 
-   add_file_to_fileset(jcr, file, true);
+   add_file_to_fileset(jcr, fname, true);
 
    /*
     * Restore the current context
     */
    set_incexe(jcr, old);
 
-   Dmsg1(100, "Add exclude file=%s\n", file);
+   Dmsg1(100, "Add exclude file=%s\n", fname);
 
    return bRC_OK;
 }
@@ -2211,7 +2352,7 @@ static bRC bareosAddExclude(bpContext *ctx, const char *file)
 /**
  * Let the plugin define files/directories to be excluded from the main backup.
  */
-static bRC bareosAddInclude(bpContext *ctx, const char *file)
+static bRC bareosAddInclude(bpContext *ctx, const char *fname)
 {
    JCR *jcr;
    findINCEXE *old;
@@ -2221,7 +2362,7 @@ static bRC bareosAddInclude(bpContext *ctx, const char *file)
       return bRC_Error;
    }
 
-   if (!file) {
+   if (!fname) {
       return bRC_Error;
    }
 
@@ -2242,14 +2383,14 @@ static bRC bareosAddInclude(bpContext *ctx, const char *file)
    }
 
    set_incexe(jcr, bctx->include);
-   add_file_to_fileset(jcr, file, true);
+   add_file_to_fileset(jcr, fname, true);
 
    /*
     * Restore the current context
     */
    set_incexe(jcr, old);
 
-   Dmsg1(100, "Add include file=%s\n", file);
+   Dmsg1(100, "Add include file=%s\n", fname);
 
    return bRC_OK;
 }
@@ -2358,7 +2499,7 @@ static bRC bareosCheckChanges(bpContext *ctx, struct save_pkt *sp)
    JCR *jcr;
    b_plugin_ctx *bctx;
    FF_PKT *ff_pkt;
-   bRC ret = bRC_Error;
+   bRC retval = bRC_Error;
 
    if (!is_ctx_good(ctx, jcr, bctx)) {
       goto bail_out;
@@ -2388,9 +2529,9 @@ static bRC bareosCheckChanges(bpContext *ctx, struct save_pkt *sp)
    memcpy(&ff_pkt->statp, &sp->statp, sizeof(ff_pkt->statp));
 
    if (check_changes(jcr, ff_pkt))  {
-      ret = bRC_OK;
+      retval = bRC_OK;
    } else {
-      ret = bRC_Seen;
+      retval = bRC_Seen;
    }
 
    /*
@@ -2400,8 +2541,8 @@ static bRC bareosCheckChanges(bpContext *ctx, struct save_pkt *sp)
    sp->accurate_found = ff_pkt->accurate_found;
 
 bail_out:
-   Dmsg1(100, "checkChanges=%i\n", ret);
-   return ret;
+   Dmsg1(100, "checkChanges=%i\n", retval);
+   return retval;
 }
 
 /**
@@ -2412,7 +2553,7 @@ static bRC bareosAcceptFile(bpContext *ctx, struct save_pkt *sp)
    JCR *jcr;
    FF_PKT *ff_pkt;
    b_plugin_ctx *bctx;
-   bRC ret = bRC_Error;
+   bRC retval = bRC_Error;
 
    if (!is_ctx_good(ctx, jcr, bctx)) {
       goto bail_out;
@@ -2427,17 +2568,80 @@ static bRC bareosAcceptFile(bpContext *ctx, struct save_pkt *sp)
    memcpy(&ff_pkt->statp, &sp->statp, sizeof(ff_pkt->statp));
 
    if (accept_file(ff_pkt)) {
-      ret = bRC_OK;
+      retval = bRC_OK;
    } else {
-      ret = bRC_Skip;
+      retval = bRC_Skip;
    }
 
 bail_out:
-   return ret;
+   return retval;
+}
+
+/**
+ * Manipulate the accurate seen bitmap for setting bits
+ */
+static bRC bareosSetSeenBitmap(bpContext *ctx, bool all, char *fname)
+{
+   JCR *jcr;
+   b_plugin_ctx *bctx;
+   bRC retval = bRC_Error;
+
+   if (!is_ctx_good(ctx, jcr, bctx)) {
+      goto bail_out;
+   }
+
+   if (all && fname) {
+      Dmsg0(dbglvl, "fd-plugin: API error in call to SetSeenBitmap, both all and fname set!!!\n");
+      goto bail_out;
+   }
+
+   if (all) {
+      if (accurate_mark_all_files_as_seen(jcr)) {
+         retval = bRC_OK;
+      }
+   } else if (fname) {
+      if (accurate_mark_file_as_seen(jcr, fname)) {
+         retval = bRC_OK;
+      }
+   }
+
+bail_out:
+   return retval;
+}
+
+/**
+ * Manipulate the accurate seen bitmap for clearing bits
+ */
+static bRC bareosClearSeenBitmap(bpContext *ctx, bool all, char *fname)
+{
+   JCR *jcr;
+   b_plugin_ctx *bctx;
+   bRC retval = bRC_Error;
+
+   if (!is_ctx_good(ctx, jcr, bctx)) {
+      goto bail_out;
+   }
+
+   if (all && fname) {
+      Dmsg0(dbglvl, "fd-plugin: API error in call to ClearSeenBitmap, both all and fname set!!!\n");
+      goto bail_out;
+   }
+
+   if (all) {
+      if (accurate_unmark_all_files_as_seen(jcr)) {
+         retval = bRC_OK;
+      }
+   } else if (fname) {
+      if (accurate_unmark_file_as_seen(jcr, fname)) {
+         retval = bRC_OK;
+      }
+   }
+
+bail_out:
+   return retval;
 }
 
 #ifdef TEST_PROGRAM
-
 /* Exported variables */
 CLIENTRES *me;                        /* my resource */
 
