@@ -3,7 +3,7 @@
 
    Copyright (C) 2001-2010 Free Software Foundation Europe e.V.
    Copyright (C) 2011-2012 Planets Communications B.V.
-   Copyright (C) 2013-2013 Bareos GmbH & Co. KG
+   Copyright (C) 2013-2014 Bareos GmbH & Co. KG
 
    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version three of the GNU Affero General Public
@@ -26,8 +26,15 @@
  * Kern Sibbald, Jan MMI
  */
 
+#ifndef __FILED_H_
+#define __FILED_H_
+
 #define FILE_DAEMON 1
 #include "filed_conf.h"
+#ifdef HAVE_WIN32
+#include "vss.h"
+#endif
+#include "jcr.h"
 #include "lib/breg.h"
 #include "lib/htable.h"
 #include "lib/runscript.h"
@@ -39,5 +46,13 @@
 #include "protos.h"                   /* file daemon prototypes */
 
 extern CLIENTRES *me;                 /* "Global" Client resource */
+extern CONFIG *my_config;             /* Our Global config */
 
 void terminate_filed(int sig);
+
+/*
+ * File Daemon protocol version
+ */
+const int FD_PROTOCOL_VERSION = 54;
+
+#endif /* __FILED_H_ */
