@@ -26,6 +26,7 @@
  */
 
 #include "bareos.h"
+#include "jcr.h"
 #include "find.h"
 #include "ch.h"
 
@@ -36,7 +37,7 @@ static bool uid_set = false;
 #if defined(HAVE_WIN32)
 
 /* Imported Functions */
-extern void unix_name_to_win32(POOLMEM **win32_name, const char *name);
+extern void unix_name_to_win32(POOLMEM *&win32_name, const char *name);
 
 /* Forward referenced subroutines */
 static bool set_win32_attributes(JCR *jcr, ATTR *attr, BFILE *ofd);
@@ -385,7 +386,7 @@ bool set_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
        if (is_bopen(ofd)) {
            bclose(ofd);
        }
-       pm_strcpy(attr->ofname, "*none*");
+       pm_strcpy(attr->ofname, "*None*");
        return true;
    }
 
@@ -395,7 +396,7 @@ bool set_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
       if (is_bopen(ofd)) {
          bclose(ofd);
       }
-      pm_strcpy(attr->ofname, "*none*");
+      pm_strcpy(attr->ofname, "*None*");
       return true;
    }
 
@@ -489,7 +490,7 @@ bail_out:
       bclose(ofd);
    }
 
-   pm_strcpy(attr->ofname, "*none*");
+   pm_strcpy(attr->ofname, "*None*");
    umask(old_mask);
 
    return ok;
@@ -552,14 +553,13 @@ int encode_attribsEx(JCR *jcr, char *attribsEx, FF_PKT *ff_pkt)
       return STREAM_UNIX_ATTRIBUTES;
    }
 
-   unix_name_to_win32(&ff_pkt->sys_fname, ff_pkt->fname);
-
+   unix_name_to_win32(ff_pkt->sys_fname, ff_pkt->fname);
    if (p_GetFileAttributesExW)  {
       /**
        * Try unicode version
        */
       POOLMEM* pwszBuf = get_pool_memory(PM_FNAME);
-      make_win32_path_UTF8_2_wchar(&pwszBuf, ff_pkt->fname);
+      make_win32_path_UTF8_2_wchar(pwszBuf, ff_pkt->fname);
 
       BOOL b=p_GetFileAttributesExW((LPCWSTR)pwszBuf, GetFileExInfoStandard,
                                     (LPVOID)&atts);
