@@ -1,11 +1,11 @@
 #undef  VERSION
-#define VERSION "16.2.5"
-#define BDATE   "03 March 2017"
-#define LSMDATE "03Mar17"
+#define VERSION "16.2.6"
+#define BDATE   "02 June 2017"
+#define LSMDATE "02Jun17"
 
 #define PROG_COPYRIGHT "Copyright (C) %d-2012 Free Software Foundation Europe e.V.\n" \
                        "Copyright (C) 2013-2017 Bareos GmbH & Co. KG\n"
-#define BYEAR "2017"       /* year for copyright messages in progs */
+#define BYEAR "2017"       /* year for copyright messages in programs */
 
 /*
    BAREOS® - Backup Archiving REcovery Open Sourced
