@@ -167,12 +167,12 @@ void htable::hash_index(uint8_t *key, uint32_t keylen)
 /*
  * tsize is the estimated number of entries in the hash table
  */
-htable::htable(void *item, void *link, int tsize, int nr_pages)
+htable::htable(void *item, void *link, int tsize, int nr_pages, int nr_entries)
 {
-   init(item, link, tsize, nr_pages);
+   init(item, link, tsize, nr_pages, nr_entries);
 }
 
-void htable::init(void *item, void *link, int tsize, int nr_pages)
+void htable::init(void *item, void *link, int tsize, int nr_pages, int nr_entries)
 {
    int pwr;
    int pagesize;
@@ -190,7 +190,7 @@ void htable::init(void *item, void *link, int tsize, int nr_pages)
    mask = ~((~0) << pwr);             /* 3 bits => table size = 8 */
    rshift = 30 - pwr;                 /* Start using bits 28, 29, 30 */
    buckets = 1 << pwr;                /* Hash table size -- power of two */
-   max_items = buckets * 4;           /* Allow average 4 entries per chain */
+   max_items = buckets * nr_entries;  /* Allow average nr_entries entries per chain */
    table = (hlink **)malloc(buckets * sizeof(hlink *));
    memset(table, 0, buckets * sizeof(hlink *));
 
@@ -590,86 +590,3 @@ void htable::destroy()
    Dmsg0(100, "Done destroy.\n");
 }
 
-#ifdef TEST_PROGRAM
-struct MYJCR {
-#ifndef TEST_NON_CHAR
-   char *key;
-#else
-   uint32_t key;
-#endif
-   hlink link;
-};
-
-#ifndef TEST_SMALL_HTABLE
-#define NITEMS 5000000
-#else
-#define NITEMS 5000
-#endif
-
-int main()
-{
-   char mkey[30];
-   htable *jcrtbl;
-   MYJCR *save_jcr = NULL, *item;
-   MYJCR *jcr = NULL;
-   int count = 0;
-
-   jcrtbl = (htable *)malloc(sizeof(htable));
-
-#ifndef TEST_SMALL_HTABLE
-   jcrtbl->init(jcr, &jcr->link, NITEMS);
-#else
-   jcrtbl->init(jcr, &jcr->link, NITEMS, 128);
-#endif
-   Dmsg1(000, "Inserting %d items\n", NITEMS);
-   for (int i=0; i<NITEMS; i++) {
-#ifndef TEST_NON_CHAR
-      int len;
-      len = sprintf(mkey, "This is htable item %d", i) + 1;
-
-      jcr = (MYJCR *)jcrtbl->hash_malloc(sizeof(MYJCR));
-      jcr->key = (char *)jcrtbl->hash_malloc(len);
-      memcpy(jcr->key, mkey, len);
-#else
-      jcr = (MYJCR *)jcrtbl->hash_malloc(sizeof(MYJCR));
-      jcr->key = i;
-#endif
-      Dmsg2(100, "link=%p jcr=%p\n", jcr->link, jcr);
-
-      jcrtbl->insert(jcr->key, jcr);
-      if (i == 10) {
-         save_jcr = jcr;
-      }
-   }
-   if (!(item = (MYJCR *)jcrtbl->lookup(save_jcr->key))) {
-      printf("Bad news: %s not found.\n", save_jcr->key);
-   } else {
-#ifndef TEST_NON_CHAR
-      printf("Item 10's key is: %s\n", item->key);
-#else
-      printf("Item 10's key is: %ld\n", item->key);
-#endif
-   }
-
-   jcrtbl->stats();
-   printf("Walk the hash table:\n");
-   foreach_htable (jcr, jcrtbl) {
-#ifndef TEST_NON_CHAR
-//    printf("htable item = %s\n", jcr->key);
-#ifndef BIG_MALLOC
-      free(jcr->key);
-#endif
-#endif
-      count++;
-   }
-   printf("Got %d items -- %s\n", count, count==NITEMS?"OK":"***ERROR***");
-   printf("Calling destroy\n");
-   jcrtbl->destroy();
-
-   free(jcrtbl);
-   printf("Freed jcrtbl\n");
-
-   sm_dump(false);   /* unit test */
-
-}
-#endif
