@@ -37,6 +37,8 @@
 
 #include "bareos.h"
 
+#define HEAD_SIZE BALIGN(sizeof(struct abufhead))
+
 #ifdef HAVE_MALLOC_TRIM
 extern "C" int malloc_trim (size_t pad);
 #endif
@@ -101,9 +103,8 @@ struct abufhead {
 
 static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 
-#ifdef SMARTALLOC
 
-#define HEAD_SIZE BALIGN(sizeof(struct abufhead))
+
 
 /*
  * Special version of error reporting using a static buffer so we don't use
@@ -130,6 +131,8 @@ static void smart_alloc_msg(const char *file, int line, const char *fmt, ...)
    p[0] = 0;                    /* Generate segmentation violation */
 }
 
+
+#ifdef SMARTALLOC
 POOLMEM *sm_get_pool_memory(const char *fname, int lineno, int pool)
 {
    struct abufhead *buf;
@@ -285,7 +288,7 @@ POOLMEM *get_pool_memory(int pool)
       return (POOLMEM *)((char *)buf + HEAD_SIZE);
    }
 
-   if ((buf=malloc(pool_ctl[pool].size + HEAD_SIZE)) == NULL) {
+   if ((buf = (struct abufhead *)malloc(pool_ctl[pool].size + HEAD_SIZE)) == NULL) {
       V(mutex);
       smart_alloc_msg(__FILE__, __LINE__, _("Out of memory requesting %d bytes\n"), pool_ctl[pool].size);
       return NULL;
@@ -308,7 +311,7 @@ POOLMEM *get_memory(int32_t size)
    struct abufhead *buf;
    int pool = 0;
 
-   if ((buf=malloc(size + HEAD_SIZE)) == NULL) {
+   if ((buf = (struct abufhead *)malloc(size + HEAD_SIZE)) == NULL) {
       smart_alloc_msg(__FILE__, __LINE__, _("Out of memory requesting %d bytes\n"), size);
       return NULL;
    }
@@ -511,21 +514,8 @@ void print_memory_pool_stats() {}
 
 /*
  * Concatenate a string (str) onto a pool memory buffer pm
- *   Returns: length of concatenated string
+ * Returns: length of concatenated string
  */
-int pm_strcat(POOLMEM **pm, const char *str)
-{
-   int pmlen = strlen(*pm);
-   int len;
-
-   if (!str) str = "";
-
-   len = strlen(str) + 1;
-   *pm = check_pool_memory_size(*pm, pmlen + len);
-   memcpy(*pm+pmlen, str, len);
-   return pmlen + len - 1;
-}
-
 int pm_strcat(POOLMEM *&pm, const char *str)
 {
    int pmlen = strlen(pm);
@@ -562,22 +552,23 @@ int pm_strcat(POOL_MEM &pm, const char *str)
    return pmlen + len - 1;
 }
 
-/*
- * Copy a string (str) into a pool memory buffer pm
- *   Returns: length of string copied
- */
-int pm_strcpy(POOLMEM **pm, const char *str)
+int pm_strcat(POOL_MEM *&pm, const char *str)
 {
+   int pmlen = strlen(pm->c_str());
    int len;
 
    if (!str) str = "";
 
    len = strlen(str) + 1;
-   *pm = check_pool_memory_size(*pm, len);
-   memcpy(*pm, str, len);
-   return len - 1;
+   pm->check_size(pmlen + len);
+   memcpy(pm->c_str()+pmlen, str, len);
+   return pmlen + len - 1;
 }
 
+/*
+ * Copy a string (str) into a pool memory buffer pm
+ * Returns: length of string copied
+ */
 int pm_strcpy(POOLMEM *&pm, const char *str)
 {
    int len;
@@ -611,17 +602,22 @@ int pm_strcpy(POOL_MEM &pm, const char *str)
    return len - 1;
 }
 
-/*
- * Copy data into a pool memory buffer pm
- *   Returns: length of data copied
- */
-int pm_memcpy(POOLMEM **pm, const char *data, int32_t n)
+int pm_strcpy(POOL_MEM *&pm, const char *str)
 {
-   *pm = check_pool_memory_size(*pm, n);
-   memcpy(*pm, data, n);
-   return n;
+   int len;
+
+   if (!str) str = "";
+
+   len = strlen(str) + 1;
+   pm->check_size(len);
+   memcpy(pm->c_str(), str, len);
+   return len - 1;
 }
 
+/*
+ * Copy data into a pool memory buffer pm
+ * Returns: length of data copied
+ */
 int pm_memcpy(POOLMEM *&pm, const char *data, int32_t n)
 {
    pm = check_pool_memory_size(pm, n);
@@ -643,6 +639,13 @@ int pm_memcpy(POOL_MEM &pm, const char *data, int32_t n)
    return n;
 }
 
+int pm_memcpy(POOL_MEM *&pm, const char *data, int32_t n)
+{
+   pm->check_size(n);
+   memcpy(pm->c_str(), data, n);
+   return n;
+}
+
 /* ==============  CLASS POOL_MEM   ============== */
 
 /*
@@ -683,31 +686,56 @@ void POOL_MEM::realloc_pm(int32_t size)
    V(mutex);
 }
 
+int POOL_MEM::strcat(POOL_MEM &str)
+{
+   return strcat(str.c_str());
+}
+
 int POOL_MEM::strcat(const char *str)
 {
-   int pmlen = strlen(mem);
+   int pmlen = strlen();
    int len;
 
    if (!str) str = "";
 
-   len = strlen(str) + 1;
+   len = ::strlen(str) + 1;
    check_size(pmlen + len);
    memcpy(mem+pmlen, str, len);
    return pmlen + len - 1;
 }
 
+int POOL_MEM::strcpy(POOL_MEM &str)
+{
+   return strcpy(str.c_str());
+}
+
 int POOL_MEM::strcpy(const char *str)
 {
    int len;
 
    if (!str) str = "";
 
-   len = strlen(str) + 1;
+   len = ::strlen(str) + 1;
    check_size(len);
    memcpy(mem, str, len);
    return len - 1;
 }
 
+void POOL_MEM::toLower()
+{
+   lcase(mem);
+}
+
+int POOL_MEM::bsprintf(const char *fmt, ...)
+{
+   int len;
+   va_list arg_ptr;
+   va_start(arg_ptr, fmt);
+   len = bvsprintf(fmt, arg_ptr);
+   va_end(arg_ptr);
+   return len;
+}
+
 #ifdef HAVE_VA_COPY
 int POOL_MEM::bvsprintf(const char *fmt, va_list arg_ptr)
 {
