@@ -85,7 +85,6 @@ static bool do_unfill();
 
 /* Static variables */
 
-#define CONFIG_FILE "bareos-sd.conf"
 #define MAX_CMD_ARGS 30
 
 static POOLMEM *cmd;
@@ -253,10 +252,6 @@ int main(int margc, char *margv[])
       init_signals(terminate_btape);
    }
 
-   if (configfile == NULL) {
-      configfile = bstrdup(CONFIG_FILE);
-   }
-
    daemon_start_time = time(NULL);
 
    my_config = new_config_parser();
@@ -264,7 +259,7 @@ int main(int margc, char *margv[])
 
    if (DirectorName) {
       foreach_res(director, R_DIRECTOR) {
-         if (bstrcmp(director->hdr.name, DirectorName)) {
+         if (bstrcmp(director->name(), DirectorName)) {
             break;
          }
       }
@@ -372,7 +367,7 @@ static void terminate_btape(int status)
    close_memory_pool();               /* free memory in pool */
    lmgr_cleanup_main();
 
-   sm_dump(false);
+   sm_dump(false, false);
    exit(status);
 }
 
@@ -690,41 +685,40 @@ static void bsrcmd()
 }
 
 /*
- * List device capabilities as defined in the
- *  stored.conf file.
+ * List device capabilities as defined in the stored.conf file.
  */
 static void capcmd()
 {
    printf(_("Configured device capabilities:\n"));
-   printf("%sEOF ", dev->capabilities & CAP_EOF ? "" : "!");
-   printf("%sBSR ", dev->capabilities & CAP_BSR ? "" : "!");
-   printf("%sBSF ", dev->capabilities & CAP_BSF ? "" : "!");
-   printf("%sFSR ", dev->capabilities & CAP_FSR ? "" : "!");
-   printf("%sFSF ", dev->capabilities & CAP_FSF ? "" : "!");
-   printf("%sFASTFSF ", dev->capabilities & CAP_FASTFSF ? "" : "!");
-   printf("%sBSFATEOM ", dev->capabilities & CAP_BSFATEOM ? "" : "!");
-   printf("%sEOM ", dev->capabilities & CAP_EOM ? "" : "!");
-   printf("%sREM ", dev->capabilities & CAP_REM ? "" : "!");
-   printf("%sRACCESS ", dev->capabilities & CAP_RACCESS ? "" : "!");
-   printf("%sAUTOMOUNT ", dev->capabilities & CAP_AUTOMOUNT ? "" : "!");
-   printf("%sLABEL ", dev->capabilities & CAP_LABEL ? "" : "!");
-   printf("%sANONVOLS ", dev->capabilities & CAP_ANONVOLS ? "" : "!");
-   printf("%sALWAYSOPEN ", dev->capabilities & CAP_ALWAYSOPEN ? "" : "!");
-   printf("%sMTIOCGET ", dev->capabilities & CAP_MTIOCGET ? "" : "!");
+   printf("%sEOF ", dev->has_cap(CAP_EOF) ? "" : "!");
+   printf("%sBSR ", dev->has_cap(CAP_BSR) ? "" : "!");
+   printf("%sBSF ", dev->has_cap(CAP_BSF) ? "" : "!");
+   printf("%sFSR ", dev->has_cap(CAP_FSR) ? "" : "!");
+   printf("%sFSF ", dev->has_cap(CAP_FSF) ? "" : "!");
+   printf("%sFASTFSF ", dev->has_cap(CAP_FASTFSF) ? "" : "!");
+   printf("%sBSFATEOM ", dev->has_cap(CAP_BSFATEOM) ? "" : "!");
+   printf("%sEOM ", dev->has_cap(CAP_EOM) ? "" : "!");
+   printf("%sREM ", dev->has_cap(CAP_REM) ? "" : "!");
+   printf("%sRACCESS ", dev->has_cap(CAP_RACCESS) ? "" : "!");
+   printf("%sAUTOMOUNT ", dev->has_cap(CAP_AUTOMOUNT) ? "" : "!");
+   printf("%sLABEL ", dev->has_cap(CAP_LABEL) ? "" : "!");
+   printf("%sANONVOLS ", dev->has_cap(CAP_ANONVOLS) ? "" : "!");
+   printf("%sALWAYSOPEN ", dev->has_cap(CAP_ALWAYSOPEN) ? "" : "!");
+   printf("%sMTIOCGET ", dev->has_cap(CAP_MTIOCGET) ? "" : "!");
    printf("\n");
 
    printf(_("Device status:\n"));
    printf("%sOPENED ", dev->is_open() ? "" : "!");
    printf("%sTAPE ", dev->is_tape() ? "" : "!");
    printf("%sLABEL ", dev->is_labeled() ? "" : "!");
-   printf("%sMALLOC ", dev->state & ST_MALLOC ? "" : "!");
+   printf("%sMALLOC ", bit_is_set(ST_MALLOC, dev->state) ? "" : "!");
    printf("%sAPPEND ", dev->can_append() ? "" : "!");
    printf("%sREAD ", dev->can_read() ? "" : "!");
    printf("%sEOT ", dev->at_eot() ? "" : "!");
-   printf("%sWEOT ", dev->state & ST_WEOT ? "" : "!");
+   printf("%sWEOT ", bit_is_set(ST_WEOT, dev->state) ? "" : "!");
    printf("%sEOF ", dev->at_eof() ? "" : "!");
-   printf("%sNEXTVOL ", dev->state & ST_NEXTVOL ? "" : "!");
-   printf("%sSHORT ", dev->state & ST_SHORT ? "" : "!");
+   printf("%sNEXTVOL ", bit_is_set(ST_NEXTVOL, dev->state) ? "" : "!");
+   printf("%sSHORT ", bit_is_set(ST_SHORT, dev->state) ? "" : "!");
    printf("\n");
 
    printf(_("Device parameters:\n"));
@@ -798,7 +792,7 @@ static bool re_read_block_test()
    bool rc = false;
    int len;
 
-   if (!(dev->capabilities & CAP_BSR)) {
+   if (!dev->has_cap(CAP_BSR)) {
       Pmsg0(-1, _("Skipping read backwards test because BSR turned off.\n"));
       return true;
    }
@@ -1122,9 +1116,8 @@ static bool write_two_files()
 {
    DEV_BLOCK *block;
    DEV_RECORD *rec;
-   int len, j;
-   unsigned int i;
-   int *p;
+   uint32_t len;
+   uint32_t *p;
    bool rc = false;       /* bad return code */
    DEVICE *dev = dcr->dev;
 
@@ -1150,16 +1143,16 @@ static bool write_two_files()
    rec = new_record();
    rec->data = check_pool_memory_size(rec->data, block->buf_len);
    rec->data_len = block->buf_len-100;
-   len = rec->data_len/sizeof(i);
+   len = rec->data_len / sizeof(uint32_t);
 
    if (!dev->rewind(dcr)) {
       Pmsg1(0, _("Bad status from rewind. ERR=%s\n"), dev->bstrerror());
       goto bail_out;
    }
 
-   for (i=1; i<=num_recs; i++) {
-      p = (int *)rec->data;
-      for (j=0; j<len; j++) {
+   for (uint32_t i = 1; i <= num_recs; i++) {
+      p = (uint32_t *)rec->data;
+      for (uint32_t j = 0; j < len; j++) {
          *p++ = i;
       }
       if (!write_record_to_block(dcr, rec)) {
@@ -1173,9 +1166,9 @@ static bool write_two_files()
    }
    Pmsg2(0, _("Wrote %d blocks of %d bytes.\n"), num_recs, rec->data_len);
    weofcmd();
-   for (i=num_recs+1; i<=2*num_recs; i++) {
-      p = (int *)rec->data;
-      for (j=0; j<len; j++) {
+   for (uint32_t i = num_recs + 1; i <= 2 * num_recs; i++) {
+      p = (uint32_t *)rec->data;
+      for (uint32_t j = 0; j < len; j++) {
          *p++ = i;
       }
       if (!write_record_to_block(dcr, rec)) {
@@ -1199,22 +1192,22 @@ bail_out:
    if (!rc) {
       exit_code = 1;
    }
-   return rc;
 
+   return rc;
 }
 
 /*
  * This test writes Bareos blocks to the tape in
- *   several files. It then rewinds the tape and attepts
- *   to read these blocks back checking the data.
+ * several files. It then rewinds the tape and attepts
+ * to read these blocks back checking the data.
  */
 static bool write_read_test()
 {
    DEV_BLOCK *block;
    DEV_RECORD *rec;
    bool rc = false;
-   int len, i, j;
-   int *p;
+   uint32_t len;
+   uint32_t *p;
 
    rec = new_record();
 
@@ -1233,11 +1226,13 @@ static bool write_read_test()
    }
 
    rec->data = check_pool_memory_size(rec->data, block->buf_len);
-   rec->data_len = block->buf_len-100;
-   len = rec->data_len/sizeof(i);
+   rec->data_len = block->buf_len - 100;
+   len = rec->data_len / sizeof(uint32_t);
 
-   /* Now read it back */
-   for (i=1; i<=2*num_recs; i++) {
+   /*
+    * Now read it back
+    */
+   for (uint32_t i = 1; i <= 2 * num_recs; i++) {
 read_again:
       if (!dcr->read_block_from_dev(NO_BLOCK_NUMBER_CHECK)) {
          berrno be;
@@ -1256,16 +1251,15 @@ read_again:
          Pmsg2(0, _("Read record failed. Block %d! ERR=%s\n"), i, be.bstrerror(dev->dev_errno));
          goto bail_out;
       }
-      p = (int *)rec->data;
-      for (j=0; j<len; j++) {
+      p = (uint32_t *)rec->data;
+      for (uint32_t j = 0; j < len; j++) {
          if (*p != i) {
-            Pmsg3(0, _("Bad data in record. Expected %d, got %d at byte %d. Test failed!\n"),
-               i, *p, j);
+            Pmsg3(0, _("Bad data in record. Expected %d, got %d at byte %d. Test failed!\n"), i, *p, j);
             goto bail_out;
          }
          p++;
       }
-      if (i == num_recs || i == 2*num_recs) {
+      if (i == num_recs || i == 2 * num_recs) {
          Pmsg1(-1, _("%d blocks re-read correctly.\n"), num_recs);
       }
    }
@@ -1282,8 +1276,8 @@ bail_out:
 
 /*
  * This test writes Bareos blocks to the tape in
- *   several files. It then rewinds the tape and attepts
- *   to read these blocks back checking the data.
+ * several files. It then rewinds the tape and attepts
+ * to read these blocks back checking the data.
  */
 static bool position_test()
 {
@@ -1477,7 +1471,8 @@ static int append_test()
 static int autochanger_test()
 {
    POOLMEM *results, *changer;
-   int slot, status, loaded;
+   slot_number_t slot, loaded;
+   int status;
    int timeout = dcr->device->max_changer_wait;
    int sleep_time = 0;
 
@@ -1533,9 +1528,9 @@ try_again:
       /* We are going to load a new tape, so close the device */
       dev->close(dcr);
       Pmsg2(-1, _("3302 Issuing autochanger \"unload %d %d\" command.\n"),
-         loaded, dev->drive_index);
+            loaded, dev->drive);
       changer = edit_device_codes(dcr, changer,
-                   dcr->device->changer_command, "unload");
+                                  dcr->device->changer_command, "unload");
       status = run_program(changer, timeout, results);
       Pmsg2(-1, _("unload status=%s %d\n"), status==0?_("OK"):_("Bad"), status);
       if (status != 0) {
@@ -1552,15 +1547,15 @@ try_again:
    slot = 1;
    dcr->VolCatInfo.Slot = slot;
    Pmsg2(-1, _("3303 Issuing autochanger \"load %d %d\" command.\n"),
-      slot, dev->drive_index);
+         slot, dev->drive);
    changer = edit_device_codes(dcr, changer,
-                dcr->device->changer_command, "load");
+                               dcr->device->changer_command, "load");
    Dmsg1(100, "Changer=%s\n", changer);
    dev->close(dcr);
    status = run_program(changer, timeout, results);
    if (status == 0) {
       Pmsg2(-1,  _("3303 Autochanger \"load %d %d\" status is OK.\n"),
-         slot, dev->drive_index);
+            slot, dev->drive);
    } else {
       berrno be;
       Pmsg1(-1, _("3993 Bad autochanger command: %s\n"), changer);
@@ -1775,7 +1770,7 @@ static void testcmd()
          if (status == -2) {
             Pmsg0(-1, _("\n\nIt looks like the append failed. Attempting again.\n"
                      "Setting \"BSF at EOM = yes\" and retrying append test.\n"));
-            dev->capabilities |= CAP_BSFATEOM; /* backspace on eom */
+            dev->set_cap(CAP_BSFATEOM); /* Backspace on eom */
             status = append_test();
             if (status == 1) {
                Pmsg0(-1, _("\n\nIt looks like the test worked this time, please add:\n\n"
@@ -2096,7 +2091,7 @@ static void scan_blocks()
             printf(_("End of File mark.\n"));
             continue;
          }
-         if (dev->state & ST_SHORT) {
+         if (bit_is_set(ST_SHORT, dev->state)) {
             if (blocks > 0) {
                if (blocks==1) {
                   printf(_("1 block of %d bytes in file %d\n"), block_size, dev->file);
@@ -2150,18 +2145,47 @@ bail_out:
       edit_uint64_with_commas(bytes, ec1));
 }
 
-
 static void statcmd()
 {
-   int debug = debug_level;
-   debug_level = 30;
-   Pmsg2(0, _("Device status: %u. ERR=%s\n"), dev->status_dev(), dev->bstrerror());
 #ifdef xxxx
-   dump_volume_label(dev);
+   int debug = debug_level;
 #endif
-   debug_level = debug;
-}
+   char *status;
 
+   status = dev->status_dev();
+
+   printf( _("Device status:"));
+   if (bit_is_set(BMT_TAPE, status))
+      printf( " TAPE");
+   if (bit_is_set(BMT_EOF, status))
+      printf( " EOF");
+   if (bit_is_set(BMT_BOT, status))
+      printf( " BOT");
+   if (bit_is_set(BMT_EOT, status))
+      printf( " EOT");
+   if (bit_is_set(BMT_SM, status))
+      printf( " SETMARK");
+   if (bit_is_set(BMT_EOD, status))
+      printf( " EOD");
+   if (bit_is_set(BMT_WR_PROT, status))
+      printf( " WRPROT");
+   if (bit_is_set(BMT_ONLINE, status))
+      printf( " ONLINE");
+   if (bit_is_set(BMT_DR_OPEN, status))
+      printf( " DOOROPEN");
+   if (bit_is_set(BMT_IM_REP_EN, status))
+      printf( " IMMREPORT");
+
+   free(status);
+
+   printf(_(". ERR=%s\n"), dev->bstrerror());
+
+#ifdef xxxx
+   debug_level = 30;
+   dump_volume_label(dev);
+   debug_level = debug;
+#endif
+}
 
 /*
  * First we label the tape, then we fill
@@ -2954,7 +2978,7 @@ do_tape_cmds()
    while (!quit && get_cmd("*")) {
       Dsm_check(200);
       found = false;
-      parse_args(cmd, &args, &argc, argk, argv, MAX_CMD_ARGS);
+      parse_args(cmd, args, &argc, argk, argv, MAX_CMD_ARGS);
       for (i=0; i<comsize; i++)       /* search for command */
          if (argc > 0 && fstrsch(argk[0],  commands[i].key)) {
             (*commands[i].func)();    /* go execute command */
@@ -2985,7 +3009,7 @@ PROG_COPYRIGHT
 "\nVersion: %s (%s)\n\n"
 "Usage: btape <options> <device_name>\n"
 "       -b <file>     specify bootstrap file\n"
-"       -c <file>     set Storage configuration file to file\n"
+"       -c <path>     specify Storage configuration file or directory\n"
 "       -D <director> specify a director name specified in the Storage\n"
 "                     configuration file for the Key Encryption Key selection\n"
 "       -d <nn>       set debug level to <nn>\n"
