@@ -50,10 +50,11 @@ bool authenticate_with_filedaemon(JCR *jcr);
 bool init_autochangers();
 int autoload_device(DCR *dcr, int writing, BSOCK *dir);
 bool autochanger_cmd(DCR *dcr, BSOCK *dir, const char *cmd);
-bool autochanger_transfer_cmd(DCR *dcr, BSOCK *dir, int src_slot, int dst_slot);
-bool unload_autochanger(DCR *dcr, int loaded);
-bool unload_dev(DCR *dcr, DEVICE *dev);
-int get_autochanger_loaded_slot(DCR *dcr);
+bool autochanger_transfer_cmd(DCR *dcr, BSOCK *dir,
+                              slot_number_t src_slot, slot_number_t dst_slot);
+bool unload_autochanger(DCR *dcr, slot_number_t loaded, bool lock_set = false);
+bool unload_dev(DCR *dcr, DEVICE *dev, bool lock_set = false);
+slot_number_t get_autochanger_loaded_slot(DCR *dcr, bool lock_set = false);
 
 /* block.c */
 void dump_block(DEV_BLOCK *b, const char *msg);
@@ -97,8 +98,8 @@ void set_new_file_parameters(DCR *dcr);
 BSR *position_device_to_first_file(JCR *jcr, DCR *dcr);
 bool try_device_repositioning(JCR *jcr, DEV_RECORD *rec, DCR *dcr);
 
-/* dircmd.c */
-void *handle_connection_request(void *arg);
+/* dir_cmd.c */
+void *handle_director_connection(BSOCK *dir);
 
 /* fd_cmds.c */
 void *handle_filed_connection(BSOCK *fd, char *job_name);
@@ -110,7 +111,6 @@ void stored_free_jcr(JCR *jcr);
 
 /* label.c */
 int read_dev_volume_label(DCR *dcr);
-void create_session_label(DCR *dcr, DEV_RECORD *rec, int label);
 void create_volume_label(DEVICE *dev, const char *VolName, const char *PoolName);
 
 #define ANSI_VOL_LABEL 0
@@ -152,7 +152,7 @@ bool mount_next_read_volume(DCR *dcr);
 /* ndmp_tape.c */
 void end_of_ndmp_backup(JCR *jcr);
 void end_of_ndmp_restore(JCR *jcr);
-int start_ndmp_thread_server(dlist *addr_list, int max_clients, workq_t *client_wq);
+int start_ndmp_thread_server(dlist *addr_list, int max_clients);
 void stop_ndmp_thread_server();
 
 /* parse_bsr.c */
@@ -191,14 +191,15 @@ bool read_records(DCR *dcr,
 /* record.c */
 const char *FI_to_ascii(char *buf, int fi);
 const char *stream_to_ascii(char *buf, int stream, int fi);
+void dump_record(const char *tag, const DEV_RECORD *rec);
 bool write_record_to_block(DCR *dcr, DEV_RECORD *rec);
-bool can_write_record_to_block(DEV_BLOCK *block, DEV_RECORD *rec);
+bool can_write_record_to_block(DEV_BLOCK *block, const DEV_RECORD *rec);
 bool read_record_from_block(DCR *dcr, DEV_RECORD *rec);
 DEV_RECORD *new_record(bool with_data = true);
 void empty_record(DEV_RECORD *rec);
 void copy_record_state(DEV_RECORD *dst, DEV_RECORD *src);
 void free_record(DEV_RECORD *rec);
-uint64_t get_record_address(DEV_RECORD *rec);
+uint64_t get_record_address(const DEV_RECORD *rec);
 
 /* reserve.c */
 void init_reservations_lock();
@@ -259,7 +260,7 @@ void *handle_stored_connection(BSOCK *sd, char *job_name);
 bool do_listen_run(JCR *jcr);
 
 /* sd_plugins.c */
-char *edit_device_codes(DCR *dcr, char *omsg, const char *imsg, const char *cmd);
+char *edit_device_codes(DCR *dcr, POOLMEM *&omsg, const char *imsg, const char *cmd);
 
 /* sd_stats.c */
 int start_statistics_thread(void);
@@ -267,6 +268,10 @@ void stop_statistics_thread();
 void update_device_tapealert(const char *devname, uint64_t flags, utime_t now);
 void update_job_statistics(JCR *jcr, utime_t now);
 
+/* socket_server.c */
+void start_socket_server(dlist *addrs);
+void stop_socket_server();
+
 /* spool.c */
 bool begin_data_spool (DCR *dcr);
 bool discard_data_spool (DCR *dcr);
