@@ -28,5 +28,4 @@
 #define SERVICE_DESC "Provides file backup and restore services (bareos client)."
 
 #define terminate_app(x) terminate_filed(x)
-extern void terminate_filed(int sig);
-extern void VSSInit();
+void terminate_filed(int sig);
