@@ -26,6 +26,7 @@
 
 #if defined(HAVE_WIN32)
 
+#include "jcr.h"
 #include "find.h"
 #include "lib/cbuf.h"
 
@@ -190,7 +191,7 @@ int get_win32_virtualmountpoints(findFILESET *fileset, dlist **szVmps)
                continue;
             }
 
-            if (win32_get_vmp_devicename(fname, &devicename)) {
+            if (win32_get_vmp_devicename(fname, devicename)) {
                /*
                 * See if we need to allocate a new dlist.
                 */
